package jp.co.cam.usax.license;

import java.net.InetAddress;
import java.sql.Date;

// commons
import jp.co.cam.usax.LicenseManager;

// rules
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.impl.AbstractScreenEvent;
import jp.co.cam.usax.web.WebAnnotations.PageScope;
import jp.co.cam.usax.web.WebException;


/**
 * 【Universal X】ライセンス認証画面。
 * <br>
 * <br>Copyright(c) 1999-2016 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2016
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>
 * <br>########################################################################
 */
@PageScope("request")
public class LicenseEvent
    extends AbstractScreenEvent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2017L;


    @Override
    public void page_load()
        throws RuleException
    {
        // 起動開始日
        final Date install;

        // 稼働日数
        long days;

        try {

            super.setString("HOST_NAME", InetAddress.getLocalHost().getHostName());

            install = new Date(LicenseManager.getInstallDate().getTime());
            super.setDate("INSTALLED", install);

            days = (System.currentTimeMillis() - install.getTime()) / 1000 / 60 / 60 /24;

            super.setInteger("USED_DAYS", ((int)days));

        } catch (final Exception ex) {
            throw new WebException(ex, "ライセンスの状況が確認できません");
        }
    }

    /**
     * ライセンス登録。
     * @throws WebException 発生した例外を報告
     */
    public void btn_regist_event()
        throws WebException
    {
        try {
            
            LicenseManager.setLicenseCode(super.getString("LICENSE"));

            if (!LicenseManager.checkLicenseCode(super.getString("HOST_NAME"))) {
                super.getMessage().addError("ライセンス・コードが正しくありません");
                LicenseManager.setLicenseCode(null);
                return;
            }

            super.getPageCtrl().addRewrite("BLOCK_DETAIL");

        } catch (final Exception ex) {
            throw new WebException(ex, "ライセンス確認に失敗");
        }
    }
}
