package jp.co.hogehoge;

import java.util.Enumeration;
import java.util.Map;

import javax.servlet.http.HttpSession;

// rules
import jp.co.cam.usax.USAXAnnotations.UnSupportedAttribute;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.RuleException;


/**
 * 【Example】イベントクラスの基底クラスのサンプル。
 * <br>このクラスはスクリーン・イベントを作成する際の共通処理を実装する
 * <br>基底クラスのサンプルです。
 * <br>本クラスを参考に各会社様ごとに必要な処理を実装してください。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@UnSupportedAttribute
public abstract class AbstractExampleScreenEvent
    extends UniversalInspectionScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    /**
     * 【共通処理】組織切替え処理。
     */
    public void change_corporate_event()
    {
        // 組織切替え画面への遷移
        this.setNextPage(new CorporateChangeEvent());
    }

    /**
     * 【共通処理】ホーム画面へ遷移。
     */
    public void btn_home_event()
    {
        // 組織切替え画面への遷移
        this.setNextPage(new HomeEvent());
    }

    /**
     * 【共通処理】ログアウト処理。
     */
    public void signout_event()
    {
        // Session
        final HttpSession session = super.getUserIO().getSession();

        // キー名一覧
        final Enumeration<String> keys
         = session.getAttributeNames();

        while (keys.hasMoreElements())
        {
            session.removeAttribute(keys.nextElement());
        }

        // ログイン画面への遷移
        this.setNextPage(new SignEvent());
    }


    /**
     * ログを記録。
     * @param  pmType ログ種別
     * @param  pmText ログ
     * @throws RuleException 処理中に発生した例外を報告
     */
    public void log(
        final String pmType
      , final String pmText
    )
        throws RuleException
    {
        // ルール：ログの記録
        final RuleEngineIF rule = super.getRuleEngine("Log");

        // パラメータ
        final Map<String, Object> parm = rule.getParameter();

        parm.put("LOG_TYPE", pmType);
        parm.put("LOG_TEXT", pmText);

        rule.run();
    }
}
