package jp.co.hogehoge;

// rules
import jp.co.cam.usax.USAXAnnotations.UnSupportedAttribute;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.util.KeyUtil;

// web
import jp.co.cam.usax.web.event.impl.AbstractSubEvent;
import jp.co.cam.usax.web.event.io.ScreenControllerIF;


/**
 * 【Example】サブ・イベントクラスの基底クラス。
 * <br>このクラスは一覧の１行イベントを作成する際の共通処理を実装する
 * <br>基底クラスのサンプルです。
 * <br>本クラスを参考に各会社様ごとに必要な処理を実装してください。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@UnSupportedAttribute
public class CommonExampleSubEvent
    extends AbstractSubEvent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2016L;

    @Override
    public boolean data_command(final String pmCmd)
        throws RuleException
    {
        if (pmCmd.endsWith("$BTN_DELETE")) {
        // 明細の削除ボタンを押下

            // 親リスト
            final ListBeanIF list = super.getParentList();

            try {

                for (int i = 0; i < list.getSize(); i++)
                {
                    if (list.get(i) == this) {
                        list.del(i);
                        break;
                    }
                }

            } catch (final Exception ex) {
                throw new RuleException(ex, "行削除に失敗しました");
            }

        } else
        if (pmCmd.endsWith("$UP")) {
        // 明細の[↑]ボタン

            // ====================================================================================
            // 順の入替え処理
            // ====================================================================================

            // 親リスト
            final ListBeanIF list = super.getParentList();

            // 行数
            final int rows = (list == null) ? 0 : list.getSize();

            // 行
            ListBeanIF.SubListBeanIF sub;

            try {

                // ----------------------------------------------------------------------------
                // 入替え作業
                // ----------------------------------------------------------------------------
                for (int i = 0; i < rows; i++)
                {
                    if (list.get(i) == this) {

                        if (0 < i) {

                            // 対象行の行NO
                            final Integer line = this.getInteger("VIEWLINE");

                            sub = list.get(i - 1);
                            list.set(this, i - 1);
                            list.set(sub, i);

                            super.setInteger("VIEWLINE", sub.getInteger("VIEWLINE"));
                            super.updated();
                            sub.setInteger("VIEWLINE",   line);
                            sub.updated();
                        }
                        break;
                    }
                }

                // ----------------------------------------------------------------------------
                // 順振直し
                // ----------------------------------------------------------------------------
                for (int i = 0; i < rows; i++)
                {
                    sub = list.get(i);
                    sub.setInteger("VIEWLINE", 1000 + (i * 10));
                }

            } catch (final Exception ex) {
                throw new RuleException(ex, "上位置への移動に失敗しました");
            }

        } else
        if (pmCmd.endsWith("$DW")) {
        // 明細の[↓]ボタン

            // ====================================================================================
            // 順の入替え処理
            // ====================================================================================

            // 親リスト
            final ListBeanIF list = super.getParentList();

            // 行
            ListBeanIF.SubListBeanIF sub;

            try {

                for (int i = 0; i < list.getSize(); i++)
                {
                    if (list.get(i) == this) {

                        if (i + 1 < list.getSize()) {

                            // 対象行の行NO
                            final Integer line = this.getInteger("VIEWLINE");

                            sub = list.get(i + 1);
                            list.set(this, i + 1);
                            list.set(sub, i);

                            super.setInteger("VIEWLINE", sub.getInteger("VIEWLINE"));
                            super.updated();
                            sub.setInteger("VIEWLINE",   line);
                            sub.updated();
                        }
                        break;
                    }
                }

            } catch (final Exception ex) {
                throw new RuleException(ex, "下位置への移動に失敗しました");
            }
        }

        // ----------------------------------------------------------------------------------------
        // 画面の書換え命令 
        // ----------------------------------------------------------------------------------------

        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getParentEvent().getPageCtrl();

        // 書換え：一覧
        ctrl.addRewrite("BLOCK_" + KeyUtil.joinKeys(KeyUtil.parentKeys(pmCmd, 1)));

        return true;
    }

    @Override
    public void data_load()
        throws RuleException
    {
        
    }


    /**
     * コンストラクタ。
     * @param  pmList 親イベント
     * @throws RuleException 処理中に発生した例外を報告
     */
    public CommonExampleSubEvent(
        final ListBeanIF pmList
    )
        throws RuleException
    {
        super(pmList);
    }

    /**
     * コンストラクタ。
     * @throws RuleException 処理中に発生した例外を報告
     */
    public CommonExampleSubEvent()
        throws RuleException
    {
        this(null);
    }
}
