package jp.co.hogehoge;

// commons
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.WebAnnotations.PageScope;
import jp.co.cam.usax.web.event.EventIF;


/**
 * 【Example】組織切替え画面のサンプル。
 * <br>複数組織に所属するユーザーの為の組織切替え画面。
 * <br>開発難易度ならびに開発コストを考えると複数所属(兼務)対応はまだしも
 * <br>複数組織対応は権限や表示内容等が大きく異なることが容易に想像でき困難と考える。
 * <br>∴サンプルでは複数組織に所属する場合は本画面にてカレント組織を選択する仕様とする。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@PageScope("request")
@Transaction({"default"})
public class CorporateChangeEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                super.getRuleEngine("FindCorporate").run();
            }
        }
    }


    /**
     * 組織一覧用サブ・イベント。
     */
    public static class CorporateList
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            if (pmCmd.endsWith("$BTN_CORPORATE_SELECT")) {
            // ====================================================================================
            // 組織の選択
            // ====================================================================================

                // 画面イベント
                final EventIF parent = super.getParentEvent();

                // --------------------------------------------------------------------------------
                // ログの記録
                // --------------------------------------------------------------------------------
                if (parent instanceof AbstractExampleScreenEvent) {

                    // ログ
                    final String buff = "[SET CURRENT CORPORATE]"
                                      + System.getProperty("line.separator")
                                      + "CODE:" + super.getString(UserInfo.CORPORATE_CODE)
                                      + System.getProperty("line.separator")
                                      + "NAME:" + super.getString(UserInfo.CORPORATE_NAME)
                                      + System.getProperty("line.separator");

                    ((AbstractExampleScreenEvent)parent).log("LOGIN", buff);
                }

                // --------------------------------------------------------------------------------
                // 組織の選択
                // --------------------------------------------------------------------------------

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                info.setCorporateCode(super.getString(UserInfo.CORPORATE_CODE));
                info.setCorporateName(super.getString(UserInfo.CORPORATE_NAME));
                if (!info.hasAuthority(UserInfo.PUBLIC)) info.signin();

                parent.setNextPage(new HomeEvent());
            }
            return true;
        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public CorporateList(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public CorporateList()
            throws RuleException
        {
            this(null);
        }        
    }
}