package jp.co.hogehoge;

import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

// commons
import jp.co.cam.usax.configuration.ResourceFactory;
import jp.co.cam.usax.configuration.ResourceIF;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.util.text.Crypt;

// domain
import jp.co.cam.usax.appl.domain.entity.EntitySetIF;
import jp.co.cam.usax.bmc.BMCIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.MessageIF;

// web
import jp.co.cam.usax.web.WebAnnotations.PageScope;
import jp.co.cam.usax.web.WebException;
import jp.co.cam.usax.web.context.WebUserInfo;
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.UserIO;


/**
 * 【Example】ユーザー認証画面のサンプル。
 * <br>本クラスを参考に各会社様ごとに必要な処理を実装してください。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@PageScope("request")
@Transaction({"default"})
public class SignEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("処理なし");
        log.debug("============================================================");
    }


    /**
     * ログイン処理の入力チェック。
     * @return チェック結果(true 正常 or false 異常)
     */
    public boolean btn_login_check()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("ユーザー認証");
        log.debug("============================================================");
        this.clearSession();

        // パラメータ
        final Map<String, Object> parm;

        // Rule
        final RuleEngineIF rules;

        rules = super.getRuleEngine("login");
        parm = rules.getParameter();

        // パスワード
        final String pass = super.getString("PASSWORD");

        if (pass != null) {

            // パスワードの暗号化
            parm.put("CRYPT", Crypt.toHash(pass));
        }

        log.debug("------------------------------------------------------------");
        log.debug("管理者強制ログイン判定");
        log.debug("------------------------------------------------------------");
        super.setBoolean("HACK", false);

        // 基本設定
        final ResourceIF conf = ResourceFactory.get();

        // 管理者強制ログイン・パスワード
        final String admin = conf.get("security.admin");

        if (admin == null || admin.trim().length() == 0) {
            log.warning("設定", "OFF");
        } else {

            log.warning("設定", "ON");

            if (admin.equals(pass)) {
                log.warning("管理者強制ログインを実行");
                super.setBoolean("HACK", true);
                parm.put("HACK", 1);
            }
        }
        return rules.check();
    }

    /**
     * ログイン処理。
     */
    public void btn_login_event()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("ユーザー情報の取得");
        log.debug("============================================================");
        this.signin();

        log.debug("============================================================");
        log.debug("one-time-passwordの利用設定を確認");
        log.debug("============================================================");

        // 基本設定
        final ResourceIF conf = ResourceFactory.get();

        // OTPの利用有無
        final String mode;

        mode = conf.get("security.otp.using");

        if (mode == null || mode.length() == 0 || "FALSE".equals(mode.toUpperCase())) {

            log.debug("> one-time-passwordを使用しない");
            btn_finish_event();

        } else {

            log.debug("> one-time-passwordを使用する");
            btn_onetime_event();
        }
    }

    /**
     * ログイン処理のエラー処理。
     * <br>ログイン・エラーのログを記録する。
     */
    public void btn_login_error()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("ログイン失敗");
        log.debug("============================================================");

        log.debug("エラー回数の記録");
        log.debug("------------------------------------------------------------");
        super.getRuleEngine("Login.Error").run();

        log.debug("ログの記録");
        log.debug("------------------------------------------------------------");

        // ログ
        final String buff = "[ERROR]"
                          + System.getProperty("line.separator")
                          + super.getMessage()
                          + System.getProperty("line.separator");

        super.log("LOGIN", buff);
    }

    /**
     * ワンタイム認証の入力チェック。
     * @return チェック結果(true 正常 or false 異常)
     */
    public boolean btn_finish_check()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("ワンタイム認証の入力チェック");
        log.debug("============================================================");

        // 基本設定
        final ResourceIF conf = ResourceFactory.get();

        // OTPの利用有無
        final String pass;

        pass = conf.get("security.otp");

        if (pass != null && pass.length() != 0) {

            if (pass.equals(super.getString("OTP"))) {
                return true;
            }
        }

        if (super.getRuleEngine("Finish").check()) {
            return true;
        } else {

            super.addInteger("BALK_OTP", 1);

            log.debug("ワンタイム・パスワードの失効");
            log.debug("------------------------------------------------------------");
            if (3 < getInteger("BALK_OTP")) {

                super.setString("TMP_ONETIME", null);

                // メッセージ・コンテナ
                final MessageIF msg = super.getMessage();

                msg.addMessage(msg.getMessageResource().get("SGN1102"));
            }
            return false;
        }
    }

    /**
     * ワンタイム認証。
     */
    public void btn_finish_event()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("ログイン成功");
        log.debug("============================================================");

        log.debug("前回のログイン日時を取得／ログイン失敗回数のクリア");
        log.debug("------------------------------------------------------------");
        super.getRuleEngine("Login.Success").run();

        // 前回ログイン日時
        final Timestamp lastLogin = super.getDateTime("LASTLOGIN");

        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // session情報
        final Map<String, Object> session = info.getInfinityDataMap();

        if (lastLogin != null) {
            session.put("LASTLOGIN", lastLogin);
        }

        log.debug("============================================================");
        log.debug("画面遷移");
        log.debug("============================================================");

        // ログ
        final StringBuilder buff = new StringBuilder();

        buff.append("SECURITY CHECK:SUCCESS");
        buff.append(System.getProperty("line.separator"));
        buff.append(System.getProperty("line.separator"));

        if (info.hasMultiCorporate()) {
        // 組織が未選択の場合(所属組織が複数)

            log.debug("MULTI-CORPORATE");
            log.debug("------------------------------------------------------------");

            log.debug("ログの記録");
            log.debug("------------------------------------------------------------");
            super.log("LOGIN", buff.toString());

            // 認証済み権限
            final Map<String, Object> auth = new HashMap<>();

            auth.put(UserInfo.CORPORATE_CODE, "*");
            auth.put(UserInfo.AUTHORITY,      "MULTI-CORPORATE");
            info.addUserAuthority(auth);

            log.debug("組織選択画面へ遷移");
            log.debug("------------------------------------------------------------");
            super.setNextPage(new CorporateChangeEvent());

        } else {
        // 組織が選択済の場合(所属組織が単一)

            log.debug("SINGLE-CORPORATE");
            log.debug("------------------------------------------------------------");

            log.debug("ログの記録");
            log.debug("------------------------------------------------------------");
            buff.append("[SET CURRENT CORPORATE]");
            buff.append(System.getProperty("line.separator"));

            buff.append("CODE:");
            buff.append(info.getCorporateCode());
            buff.append(System.getProperty("line.separator"));
            buff.append("NAME:");
            buff.append(info.getCorporateName());
            buff.append(System.getProperty("line.separator"));

            super.log("LOGIN", buff.toString());

            info.signin();

            log.debug("ホーム画面へ遷移");
            log.debug("------------------------------------------------------------");

            super.setNextPage(new HomeEvent());
        }
    }


    /**
     * ワンタイム・パスワードの発行。
     */
    public void btn_onetime_event()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        if (!super.getBoolean("HACK")) {

            log.debug("[仕様]HackモードでのログインではOTPは発行しない");

            super.setString("TMP_ONETIME", Crypt.createPassPhrase(6));
            sendOntimePasswordMail();
        }
        super.setInteger("BALK_OTP", 1);

        this.rewrite("ワンタイム・パスワード発行モード");
    }


    /**
     * パスワード忘れた処理。
     * <br>パスワード再発行要求画面の表示。
     */
    public void btn_forgot_event()
    {
        super.setString("PASSWORD",   null);

        this.rewrite("パスワード再発行モード");
    }


    /**
     * キャンセル処理。
     * <br>１段階目の認証画面に戻る。
     */
    public void btn_cancel_event()
    {
        super.setString("PASSWORD",  null);

        this.rewrite("状態の更新");
    }


    /**
     * パスワード再発行処理。
     * @return true 成功 or false 失敗
     */
    public boolean btn_reissue_event()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("パスワードの再発行");
        log.debug("============================================================");

        // パスワード
        final String pass = Crypt.toHash(Crypt.createPassPhrase(8)).substring(0, 8).toUpperCase();

        try {

            super.setString("TMP_PASSWORD", pass);
            super.setString("TMP_CRYPT",    Crypt.toHash(pass));

            if (!super.getRuleEngine("forgot").run()) {
                return false;
            }

            // メール送信
            this.sendNewPasswordMail();

        } finally {

            super.setString("TMP_PASSWORD", null);
            super.setString("TMP_CRYPT",    null);
        }

        log.debug("============================================================");
        log.debug("メッセージの設定");
        log.debug("============================================================");

        // メッセージ
        final MessageIF msg = super.getMessage();

        msg.addMessage(msg.getMessageResource().get("SGN1008"));

        log.debug("============================================================");
        log.debug("画面再描画");
        log.debug("============================================================");
        this.rewrite("状態の更新");

        return true;
    }


    /**
     * 認証。
     */
    public void signin()
    {
        // ログAPI
        final DebuggerIF log
         = DebuggerFactory.get();

        log.debug("========================================================");
        log.debug("ユーザー情報の初期化");
        log.debug("========================================================");
        super.getRuleEngine("GetUserInformation").run();

        // ユーザー情報
        final UserInfo user = UserInfo.getUserInfo();

            user.setUserCode(super.getString(UserInfo.USER_CODE));
            user.setUserName(super.getString(UserInfo.USER_NAME));
            user.setUserEMail(super.getString(UserInfo.USER_EMAIL));
            user.setUserPhone(super.getString(UserInfo.USER_PHONE));

        try (final EntitySetIF set = (EntitySetIF)super.getObject("UserAuthority")) {
            user.setUserAuthority(set.toArray());
        } finally {
            super.setObject("UserAuthority", null);
        }

        try (final EntitySetIF set = (EntitySetIF)super.getObject("UserGroup")) {
            user.setUserGroupAuthority(set.toArray());
        } finally {
            super.setObject("UserGroup", null);
        }

        UserInfo.setUserInfo(user);

        super.getUserIO().getSession().setAttribute(UserInfo.CONTEXT_SAVE_KEY, user);
    }


    /**
     * セッション情報のクリア。
     */
    public void clearSession()
    {
        // ログAPI
        final DebuggerIF log
         = DebuggerFactory.get();

        log.debug("========================================================");
        log.debug("セッション情報のクリア");
        log.debug("========================================================");

        // ユーザーIO
        final UserIO io = super.getUserIO();

        io.removeAllSessionAttribute();

        log.debug("========================================================");
        log.debug("ユーザー情報をGuestに変換");
        log.debug("========================================================");

        // 既存のユーザー情報
        final WebUserInfo info = (WebUserInfo)UserInfo.getUserInfo();

        if (!info.isGuest()) {

            // 既存のユーザー情報の削除
            UserInfo.setUserInfo(null);

            // ゲスト情報の生成
            final WebUserInfo guest = (WebUserInfo) UserInfo.getUserInfo();

            guest.setDefaultTimezone(info.getDefaultTimezone());
            guest.setTimezone(info.getTimezone());
            guest.setClientIP(info.getClientIP());
            guest.setLanguage(info.getLanguage());
            guest.setCountry(info.getCountry());
        }
        io.save(this);
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        super.setString("ONETIME_PASSWORD", null);
        super.setString("FORGOT_PASSWORD",  null);
        ctrl.clearReadOnly();
        ctrl.clearRewrite();

        switch (pmMode)
        {
            case "ワンタイム・パスワード発行モード":

                super.setBoolean("ONETIME_PASSWORD", true);
                ctrl.addRewrite("BLOCK_DETAIL");
                break;

            case "パスワード再発行モード":

                super.setBoolean("FORGOT_PASSWORD", true);
                ctrl.addRewrite("BLOCK_DETAIL");
                break;

            case "状態の更新":
                ctrl.addRewrite("BLOCK_DETAIL");
                break;
        }
    }


    /**
     * ワンタイム・パスワード発行メールの送信。
     */
    private void sendOntimePasswordMail()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("========================================================");
        log.debug("ワンタイム・パスワード発行メールの作成");
        log.debug("========================================================");

        log.debug("メール雛形の取得");
        log.debug("--------------------------------------------------------");

        // ルールエンジン：文書の取得
        final RuleEngineIF engine = super.getRuleEngine("GetDocument");

        // パラメータ
        final Map<String, Object> parm = engine.getParameter();

        parm.put("DOC_TYPE", "MAIL_ONETIME_PASS");
        engine.run();

        try {

            log.debug("メールの編集");
            log.debug("--------------------------------------------------------");

            // メール・テンプレート
            final ListBeanIF list = super.getSubList("DOCUMENT_LIST");
    
            // メール題目
            String subject = "";
    
            // メール本文
            StringBuilder text = new StringBuilder();

            if (list.getSize() == 0) {
                throw new WebException("SYS_DOCUMENTにワンタイム・パスワード発行メールのテンプレートを登録してください");
            }

            for (final ListBeanIF.SubListBeanIF sub : list.getList())
            {
                if (subject.length() == 0) {
                    subject = sub.getString("DOC_NAME");
                }
                text.append(sub.getString("DOC_DATA"));
                text.append(System.getProperty("line.separator"));
            }

            // 本文中のパスワード記述欄に一時パスワードを設定
            text = new StringBuilder(text.toString().replace("{@PASSWORD}", super.getString("TMP_ONETIME")));

            log.debug("========================================================");
            log.debug("メールの送信");
            log.debug("========================================================");

            // BMC：メール送信
            final BMCIF mail = super.getBusinessModelByID("SendEmailBMC");

            mail.setString("EMAIL",   super.getString("USER_EMAIL"));

            mail.setString("SUBJECT", subject);
            mail.setString("TEXT",    text.toString());

            mail.run();

        } finally {

            super.setSubList("DOCUMENT_LIST", null);
        }
    }

    /**
     * パスワード再発行メールの送信。
     */
    private void sendNewPasswordMail()
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("========================================================");
        log.debug("パスワード再発行メールの作成");
        log.debug("========================================================");

        log.debug("メール雛形の取得");
        log.debug("--------------------------------------------------------");

        // ルールエンジン：文書の取得
        final RuleEngineIF engine = super.getRuleEngine("GetDocument");

        // パラメータ
        final Map<String, Object> parm = engine.getParameter();

        parm.put("DOC_TYPE", "MAIL_ISSUE_PASS");
        engine.run();

        try {

            log.debug("メールの編集");
            log.debug("--------------------------------------------------------");

            // メール・テンプレート
            final ListBeanIF list = super.getSubList("DOCUMENT_LIST");
    
            // メール題目
            String subject = "";
    
            // メール本文
            StringBuilder text = new StringBuilder();

            if (list.getSize() == 0) {
                throw new WebException("SYS_DOCUMENTにパスワード再発行メールのテンプレートを登録してください");
            }

            for (final ListBeanIF.SubListBeanIF sub : list.getList())
            {
                if (subject.length() == 0) {
                    subject = sub.getString("DOC_NAME");
                }
                text.append(sub.getString("DOC_DATA"));
                text.append(System.getProperty("line.separator"));
            }

            // 本文中のパスワード記述欄に一時パスワードを設定
            text = new StringBuilder(text.toString().replace("{@PASSWORD}", super.getString("TMP_PASSWORD")));

            log.debug("========================================================");
            log.debug("メールの送信");
            log.debug("========================================================");

            // BMC：メール送信
            final BMCIF mail = super.getBusinessModelByID("SendEmailBMC");

            mail.setString("EMAIL",   super.getString("USER_EMAIL"));

            mail.setString("SUBJECT", subject);
            mail.setString("TEXT",    text.toString());

            mail.run();

        } finally {

            super.setSubList("DOCUMENT_LIST", null);
        }
    }
}
