package jp.co.hogehoge;

// commons
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.UnSupportedAttribute;
import jp.co.cam.usax.rule.bean.MessageIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.EventIF;


/**
 * 【Example】セッション・タイムアウト処理のサンプル。
 * <br>このクラスはリクエスト送信時にセッション・タイムアウトが発生した場合の
 * <br>処理サンプルです。
 * <br>本クラスを参考に各会社様ごとに必要な処理を実装してください。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@UnSupportedAttribute
public class TimeoutEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ユーザー情報
        final UserInfo user = UserInfo.getUserInfo();

        // 遷移先
        final EventIF e;

        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        if (user.isGuest()) {
        // ユーザー情報なし

            // [仕様]下記の可能性が考えられるが、ここではセッション切れとして扱う
            // ・権限がないユーザーに機能が表示されている
            // ・不正アクセスの可能性
            // ・セッション切れ

            log.debug("★セッション・タイムアウトを発動");

            // ------------------------------------------------------------------------------------
            // エラー表示画面を初期化する ※サンプルではログイン画面にメッセージを表示
            // ------------------------------------------------------------------------------------

            // エラー表示ページ：ログイン画面
            e = new SignEvent();

            // メッセージ
            final MessageIF msg = e.getMessage();

            msg.addError(msg.getMessageResource().get("COM9000"));

            super.setNextPage(e);

        } else {
        // ユーザー情報あり(ブラウザの戻る等のアクセス時に問題発生)

            // エラー表示ページ：ホーム画面
            e = new HomeEvent();

            // メッセージ
            final MessageIF msg = e.getMessage();

            msg.addError(msg.getMessageResource().get("COM9001"));

            super.setNextPage(e);
        }
        super.getUserIO().save(e);
    }
}
