package jp.co.hogehoge;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

// commons
import jp.co.cam.usax.inspect.TestException;
import jp.co.cam.usax.util.common.Values.BValue;

// rules
import jp.co.cam.usax.USAXAnnotations.UnSupportedAttribute;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.impl.ListBean;
import jp.co.cam.usax.rule.bean.MessageIF;

// web
import jp.co.cam.usax.web.event.impl.AbstractScreenEvent;
import jp.co.cam.usax.web.event.io.UserIO;

// universal-inspection
import jp.co.cam.usax.inspect.excel.InspectionWorkBookIF;
import jp.co.cam.usax.inspect.excel.impl.InspectionWorkBook;
import jp.co.cam.usax.inspect.excel.sheet.InspectionSheetIF;
import jp.co.cam.usax.inspect.excel.sheet.InspectionSheetIF.Types;
import jp.co.cam.usax.inspect.excel.sheet.impl.OperationSheet;


/**
 * universal-inspectionの実行に対応したEventクラスのサンプル。
 * <br>このクラスはuniversal-inspectionをWEBで利用する際のサンプルです。
 * <br>本クラスを参考に各会社様ごとに必要な処理を実装してください。
 * <br>
 * <br>Copyright(c) 1999-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED ON 2020
 * <br>########################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 サンプルを再構築
 * <br>########################################################################
 */
@UnSupportedAttribute
public abstract class UniversalInspectionScreenEvent
    extends AbstractScreenEvent
{
    /** アクティブなテスト定義書 */
    private static final String USAX_INSPECTION_ACTIVE_WORKBOOK = "USAX_INSPECTION_ACTIVE_WORKBOOK";

    /** テスト中の操作シート */
    private static final String USAX_INSPECT_OPERATION = "USAX_INSPECT_OPERATION";

    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;

    /**
     * テスト定義書の設定状況を確認。
     * @return true 設定済み or false 未設定
     */
    public boolean hasInspection()
    {
        return (super.getUserIO().getSession().getAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK) != null);
    }

    /**
     * テスト定義書からシート一覧を生成する。
     * <br>作成された一覧は下記の項目名でBeanに設定
     * <br>USAX_INSPECT_DATAIMPORT_LIST データ設定
     * <br>USAX_INSPECT_OPERATION_LIST  操作
     */
    public void activeInspection()
    {
        // 操作Sheet一覧
        final ListBeanIF operationSheets;

        // ＤＢSheet一覧
        final ListBeanIF dataImportSheets;

        // テスト定義書
        final InspectionWorkBookIF book;

        book = (InspectionWorkBookIF)super.getUserIO().getSession().getAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK);

        operationSheets = new ListBean(this);
        dataImportSheets = new ListBean(this);

        for (final InspectionSheetIF sheet : book.getTestSheets())
        {
            if (sheet.getSheetType() == Types.DATA_IMPORT) {
            // データ設定

                // Sheet
                ListBeanIF.SubListBeanIF tmp = dataImportSheets.createSubBean();

                tmp.setString("value", sheet.getSheetName());
                tmp.setString("label", sheet.getSheetName());

                dataImportSheets.add(tmp);

            } else
            if (sheet.getSheetType() == Types.OPERATION) {
            // 操作

                // Sheet
                ListBeanIF.SubListBeanIF tmp = operationSheets.createSubBean();

                tmp.setString("value", sheet.getSheetName());
                tmp.setString("label", sheet.getSheetName());

                operationSheets.add(tmp);
            }
        }

        super.setSubList("USAX_INSPECT_DATAIMPORT_LIST", dataImportSheets);
        super.setSubList("USAX_INSPECT_OPERATION_LIST", operationSheets);
        super.setBoolean("USAX_INSPECT", true);
    }


    @Override
    public void page_end()
        throws RuleException
    {
        // ユーザーIO
        final UserIO io = super.getUserIO();

        // リクエスト情報
        final HttpServletRequest request;

        // セッション情報
        final HttpSession session;

        // 操作シート
        OperationSheet operation;

        if (io == null) return;

        request = io.getRequest();
        session = io.getSession();

        operation = (OperationSheet)request.getAttribute(USAX_INSPECT_OPERATION);

        if (operation == null) {
        // テスト中の操作なし

            operation = (OperationSheet)session.getAttribute(USAX_INSPECT_OPERATION);

            if (operation != null) {
            // ====================================================================================
            // 評価の実行
            // ====================================================================================

                try {

                    operation.inspection(this);

                } catch (final Exception ex) {
                    throw new RuleException(ex, "テスト定義書の読込みに失敗");
                } finally {
                    session.removeAttribute(USAX_INSPECT_OPERATION);
                }
            }

        } else {
        // テスト中の操作あり

            // 自画面で評価を実行
            request.removeAttribute(USAX_INSPECT_OPERATION);
            session.setAttribute(USAX_INSPECT_OPERATION, operation);
        }
    }

    @Override
    public boolean run()
        throws RuleException
    {
        // セッション情報
        final HttpSession session = super.getUserIO().getSession();

        // テスト定義書
        final InspectionWorkBookIF activeWorkbook
         = (InspectionWorkBookIF)session.getAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK);

        // 操作シート
        final InspectionSheetIF operation
         = (InspectionSheetIF)session.getAttribute(USAX_INSPECT_OPERATION);

        // 戻り値
        final boolean ret = super.run();

        // ========================================================================================
        // 業務ロジック中のSessionのクリアなどでテスト定義書が消えた場合は復元する
        // ========================================================================================
        if (activeWorkbook != null) {

            if (session.getAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK) == null) {
                session.setAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK, activeWorkbook);
            }
        }
        if (operation != null) {

            if (session.getAttribute(USAX_INSPECT_OPERATION) == null) {
                session.setAttribute(USAX_INSPECT_OPERATION, operation);
            }
        }
        return ret;
    }


    /**
     * テスト定義書の読込みにおける入力チェック。
     */
    public boolean btn_usax_inspection_check()
    {
        // ========================================================================================
        // テスト定義書が有効か確認する
        // ========================================================================================

        // テスト定義書
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        if (test == null || test.isClosed()) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addError("テスト定義書を指定してください");
            msg.addField("USAX_INSPECTION");

            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);
            super.setBoolean("USAX_INSPECT", false);

            return false;
        }
        return true;
    }

    /**
     * テスト定義書の読込み。
     */
    public void btn_usax_inspection_event()
    {
        // アップロード・ファイル
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        // テスト定義書
        final InspectionWorkBookIF book;

        try {

            // ====================================================================================
            // テスト定義書の読込み
            // ====================================================================================
            book = new InspectionWorkBook(test.getFile());

            // ------------------------------------------------------------------------------------
            // セッションへ保管
            // ------------------------------------------------------------------------------------
            super.getUserIO().getSession().setAttribute(USAX_INSPECTION_ACTIVE_WORKBOOK, book);

            // ------------------------------------------------------------------------------------
            // テストの有効化
            // ------------------------------------------------------------------------------------
            this.activeInspection();

            // ====================================================================================
            // 画面の再描画
            // ====================================================================================
            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);

        } catch (final Exception ex) {
            throw new RuleException(ex, "テスト定義書の読込みに失敗");
        }
    }

    /**
     * テスト定義書のデータ設定における入力チェック。
     */
    public boolean btn_usax_inspect_dataimport_check()
    {
        // メッセージ・コンテナ
        final MessageIF msg = super.getMessage();

        // ========================================================================================
        // テスト定義書が有効か確認する
        // ========================================================================================

        // テスト定義書
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        if (test == null || test.isClosed()) {

            msg.addError("テスト定義書を選択してください");
            msg.addField("USAX_INSPECTION_PATH");

            // テスト定義書を未選択状態とする
            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);
            super.setBoolean("USAX_INSPECT", false);

            super.setBlob("USAX_INSPECTION",      null);
            super.setBlob("USAX_INSPECTION_PATH", null);
            super.setBlob("USAX_INSPECTION_FILE", null);

            return false;
        }

        // ========================================================================================
        // シートが選択されているか確認する
        // ========================================================================================

        // シート名
        final String sheetName = super.getString("USAX_INSPECT_DATAIMPORT");

        if (sheetName == null) {

            msg.addError("データ設定シートを選択してください");
            msg.addField("USAX_INSPECT_DATAIMPORT");

            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);

            return false;
        }
        return true;
    }

    /**
     * テスト定義書のデータ設定。
     */
    public void btn_usax_inspect_dataimport_event()
        throws RuleException
    {
        // テスト定義書
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        try {

            // シート名
            final String sheetName = super.getString("USAX_INSPECT_DATAIMPORT");

            final InspectionWorkBookIF book = new InspectionWorkBook(test.getFile());

            for (final InspectionSheetIF sheet : book.getTestSheets())
            {
                if (sheet.getSheetName().equals(sheetName)) {

                    sheet.run();
                    break;
                }
            }

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addMessage("設定完了 - " + sheetName);

            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);
            super.setBoolean("USAX_INSPECT", true);

        } catch (final Exception ex) {
            throw new RuleException(ex, "テスト定義書の読込みに失敗");
        }
    }

    /**
     * テスト定義書の操作における入力チェック。
     */
    public boolean btn_usax_inspect_operation_check()
    {
        // メッセージ・コンテナ
        final MessageIF msg = super.getMessage();

        // ========================================================================================
        // テスト定義書が有効か確認する
        // ========================================================================================

        // テスト定義書
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        if (test == null || test.isClosed()) {

            msg.addError("テスト定義書を選択してください");
            msg.addField("USAX_INSPECTION_PATH");

            // テスト定義書を未選択状態とする
            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);
            super.setBoolean("USAX_INSPECT", false);

            super.setBlob("USAX_INSPECTION",      null);
            super.setBlob("USAX_INSPECTION_PATH", null);
            super.setBlob("USAX_INSPECTION_FILE", null);

            return false;
        }

        // ========================================================================================
        // シートが選択されているか確認する
        // ========================================================================================

        // シート名
        final String sheetName = super.getString(USAX_INSPECT_OPERATION);

        if (sheetName == null) {

            msg.addError("操作シートを選択してください");
            msg.addField(USAX_INSPECT_OPERATION);

            super.getPageCtrl().addRewrite("SCREEN_DEBUG");
            super.setEvent(UserIO.EVENT_SCREENDEBUG);

            return false;
        }

        return true;
    }

    /**
     * テスト定義書の操作。
     */
    public void btn_usax_inspect_operation_event()
        throws RuleException
    {
        // テスト定義書
        final BValue test = super.getBlob("USAX_INSPECTION_FILE");

        try {

            // テスト定義書
            final InspectionWorkBookIF book = new InspectionWorkBook(test.getFile());

            // シート名
            final String sheetName = super.getString(USAX_INSPECT_OPERATION);

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            for (final InspectionSheetIF sheet : book.getTestSheets())
            {
                if (sheet.getSheetName().equals(sheetName)) {

                    try {

                        ((OperationSheet) sheet).setEventClass(this);
                        sheet.run();

                        msg.addMessage("イベントを実行してください - " + ((OperationSheet)sheet).getEventName());
                        super.getUserIO().getRequest().setAttribute(USAX_INSPECT_OPERATION, sheet);

                    } catch (final TestException wr) {
                        msg.addMessage("テスト準備に失敗しました(テスト定義書の操作シートを確認してください) - " + sheetName);
                    }
                    super.getPageCtrl().addRewrite("BODY");
                    break;
                }
            }

        } catch (final Exception ex) {
            throw new RuleException(ex, "テスト定義書の読込みに失敗");
        }
    }
}
