package jp.co.hogehoge.bmc;

// commons
import jp.co.cam.usax.configuration.ResourceFactory;
import jp.co.cam.usax.configuration.ResourceIF;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.util.io.mail.SendMail;
import jp.co.cam.usax.USAXAnnotations;
// rules
import jp.co.cam.usax.bmc.impl.AbstractBusinessModelComponent;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;


/**
 * 【Example】BMC実装サンプル -メール送信API -。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED2016 ON none
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@USAXAnnotations.UnSupportedAttribute
public class SendEmailBMC
    extends AbstractBusinessModelComponent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public boolean run()
        throws RuleException
    {
        // メール送信API
        final SendMail mail;

        // 基本設定
        final ResourceIF conf;

        // SMTPサーバーIP
        final String smtp;

        // ポート番号
        final String port;

        // ホストIP
        final String host;

        // 送信者メールアドレス
        final String from;

        // ユーザーID
        final String user;

        // パスワード
        final String pass;

        try {

            conf = ResourceFactory.get();

            smtp = conf.get("mail.smtp.host");
            port = conf.get("mail.smtp.port");
            host = conf.get("mail.host");
            from = conf.get("mail.from");

            user = conf.get("mail.user");
            pass = conf.get("mail.pass");

            mail = new SendMail(smtp, port, host, from);

            if (user != null) {
            // SMTPへの接続にユーザー認証がある場合

                mail.setSign(user, pass);
            }

            if (super.isList("EMAIL")) {

                // 宛先一覧
                final ListBeanIF list = super.getSubList("EMAIL");

                for (final SubListBeanIF sub : list.getList())
                {
                    mail.addTO(sub.getString("EMAIL"));
                }

            } else {
                mail.addTO(super.getString("EMAIL"));
            }

            if ("TRUE".equals(conf.get("mail.send").toUpperCase())) {

                mail.setSubject(super.getString("SUBJECT"));
                mail.setBody(super.getString("TEXT"));
                mail.send();

            } else {
            // [開発中の考慮]メール送信停止中の場合は処理をスキップする

                DebuggerFactory.get().debug("メール送信の設定がOFFのため処理をスキップしました");

                // ※メールの内容はログにて参照可能です
            }

        } catch (final Exception ex) {
            throw new RuleException(ex, "メールの送信に失敗");
        }
        return true;
    }
}
