package jp.co.hogehoge.bmc.domain;

import jp.co.cam.usax.bmc.domain.impl.AbstractDomainModelComponent;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.DataBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;


/**
 * 【Example】休日情報DMC。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED2016 ON none
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
public class CalendarDMC
    extends AbstractDomainModelComponent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2020L;

    /** キー：法人ID */
    public static String PKEY_CORPORATE_CODE = "CORPORATE_CODE";


    /**
     * 休日情報を取得。
     * @throws RuleException 処理中に発生した例外を報告
     */
    public void loadHoliday()
        throws RuleException
    {
        // 返却値:休日情報
        final DataBeanIF.SubBeanIF ret = new SubBean();

        // ========================================================================================
        // 休日情報の読込み
        // ========================================================================================
        super.getRuleEngine("LoadHoliday").run();

        // ========================================================================================
        // 休日一覧の展開
        // ========================================================================================

        // 休日一覧
        final ListBeanIF holidayList = super.getSubList("HOLIDAY_LIST");

        // 休日
        SubListBeanIF holiday;

        for (int i = 0; i < holidayList.getSize(); i++)
        {
            holiday = holidayList.get(i);

            ret.setString(holiday.getString("HOLIDAY_DATE"), holiday.getString("HOLIDAY_NAME"));
        }

        super.setSubBean("HOLIDAY", ret);
    }


    /**
     * 指定された休日が存在しないことを確認。
     * @return true 指定された休日が存在しない or false 指定された休日が存在する
     * @throws RuleException 処理中に発生した例外を報告
     */
    public boolean checkDuplicate()
        throws RuleException
    {
        return super.getRuleEngine("CheckDuplicate").check();
    }
}
