package jp.co.hogehoge.bmc.domain;

import java.util.Map;

// domain
import jp.co.cam.usax.bmc.domain.impl.AbstractDomainModelComponent;

// rules
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.engine.RuleEngineIF;


/**
 * 【Example】ユーザー情報DMC。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP J.Matsuo
 * @version REVIEWED2016 ON none
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
public class UserDMC
    extends AbstractDomainModelComponent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2020L;

    /** キー：ユーザーID */
    public static String PKEY_USER_CODE = "USER_CODE";


    @Override
    public void create()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION9900").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append(" Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append(" Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.create();

        } finally {
            super.setString("LOG", null);
        }
    }

    @Override
    public void update()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION9910").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append(" Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append(" Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            if (super.getString("CRYPT") != null) {

                log.append("------------------------------");
                log.append(System.getProperty("line.separator"));
                log.append("change password");
                log.append(System.getProperty("line.separator"));
                log.append("------------------------------");
                log.append(System.getProperty("line.separator"));

                super.getRuleEngine("UpdateMyPassword").run();
            }

            super.setString("LOG", log.toString());

            super.update();

        } finally {
            super.setString("LOG", null);
        }
    }

    @Override
    public void delete()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION9999").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append(" Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append(" Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.delete();

        } finally {
            super.setString("LOG", null);
        }
    }

    /**
     * ユーザーの所属部署を登録する。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean registerAffiliation()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION2900").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" Corporate Code : ").append(super.getString("CORPORATE_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Branch Code : ").append(super.getString("BRANCH_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Group Code : ").append(super.getString("GROUP_CODE"));
        log.append(System.getProperty("line.separator"));

        if ("90".equals(super.getString("GROUP_AUTH"))) {
            log.append("Authority : ADMIN");
            log.append(System.getProperty("line.separator"));
        }

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());
    
            // エンジン：ユーザーの所属を登録
            final RuleEngineIF rule = super.getRuleEngine("RegisterAffiliation");
    
            if (rule.check()) {
    
                rule.run();
    
                return true;
    
            } else {
                return false;
            }

        } finally {
            super.setString("LOG", null);
        }
    }

    /**
     * ユーザーの所属部署を削除する。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean removeAffiliation()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION2999").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" Corporate Code : ").append(super.getString("CORPORATE_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Branch Code : ").append(super.getString("BRANCH_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Group Code : ").append(super.getString("GROUP_CODE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());
    
            // エンジン：ユーザーの所属を登録
            final RuleEngineIF rule = super.getRuleEngine("RemoveAffiliation");
    
            if (rule.check()) {
    
                rule.run();
    
                return true;
    
            } else {
                return false;
            }

        } finally {
            super.setString("LOG", null);
        }
    }

    /**
     * 所属部署における管理者権限を更新する。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean updateGroupAuthority()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION2100").getName());
        log.append(System.getProperty("line.separator"));
        log.append(" Corporate Code : ").append(super.getString("CORPORATE_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Branch Code : ").append(super.getString("BRANCH_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append(" Group Code : ").append(super.getString("GROUP_CODE"));
        log.append(System.getProperty("line.separator"));

        if ("90".equals(super.getString("GROUP_AUTH"))) {
            log.append("Authority : ADMIN");
            log.append(System.getProperty("line.separator"));
        }

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());
    
            // エンジン：ユーザーの所属部署における管理者権限を更新
            final RuleEngineIF rule = super.getRuleEngine("UpdateGroupAuthority");
    
            if (rule.check()) {
    
                rule.run();
    
                return true;
    
            } else {
                return false;
            }

        } finally {
            super.setString("LOG", null);
        }
    }

    
    /**
     * 指定された権限を追加する。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean registrationAuthority()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION1900").getName());

        log.append("Authority : ").append(super.getString("AUTHORITY"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());
    
            // エンジン：ユーザーの権限を登録する
            final RuleEngineIF rule = super.getRuleEngine("registrationAuthority");
    
            if (rule.check()) {
    
                rule.run();
    
                return true;
    
            } else {
                return false;
            }

        } finally {
            super.setString("LOG", null);
        }
    }
    
    /**
     * 指定された権限を削除する。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean removeAuthority()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION1999").getName());

        log.append("Authority : ").append(super.getString("AUTHORITY"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());
    
            // エンジン：ユーザーの権限を削除する
            final RuleEngineIF rule = super.getRuleEngine("removeAuthority");
    
            if (rule.check()) {
    
                rule.run();
    
                return true;
    
            } else {
                return false;
            }

        } finally {
            super.setString("LOG", null);
        }
    }


    /**
     * ユーザーの有効/無効を切替る。
     * @throws RuleException ルールの実行に失敗
     */
    public void changeCondition()
        throws RuleException
    {
        // ========================================================================================
        // ログの生成
        // ========================================================================================

        // ログ
        final StringBuilder log;

        if ("0".equals(super.getString("FLAG"))) {
            log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION9100").getName());
        } else {
            log = new StringBuilder(super.getAttributeResource().getAttribute("TITLE_OPERATION9199").getName());
        }
        log.append(System.getProperty("line.separator"));
        log.append(" User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append(" Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append(" Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // 標準の処理
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.getRuleEngine("ChangeCondition").run();

        } finally {
            super.setString("LOG", null);
        }
    }


    /**
     * 有効なアカウントか判定する。
     * @return true 有効 or false 無効
     * @throws RuleException ルールの実行に失敗
     */
    public boolean checkExistsAccount()
        throws RuleException
    {
        return super.getRuleEngine("CheckExistsAccount").check();
    }

    /**
     * ログイン・パスワードの確認。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean checkPassword()
        throws RuleException
    {
        // エンジン：パスワードチェック
        final RuleEngineIF rule = super.getRuleEngine("CheckPassword");

        // パラメータ・コンテナ
        final Map<String, Object> param = rule.getParameter();

        param.put("CRYPT", super.getString("CRYPT"));

        return rule.check();
    }

    /**
     * ログイン・パスワードのリセット。
     * @return true 成功 or false 失敗
     * @throws RuleException ルールの実行に失敗
     */
    public boolean resetPassword()
        throws RuleException
    {
        // ルール：ログイン・パスワードのリセット
        final RuleEngineIF rule = super.getRuleEngine("ResetPassword");

        if (rule.check()) {
            return rule.run();
        } else {
            return false;
        }
    }


    /**
     * ボーク・カウントの確認。
     * @return true 有効 or false 無効
     * @throws RuleException ルールの実行に失敗
     */
    public boolean checkBalk()
        throws RuleException
    {
        return super.getRuleEngine("CheckBalk").check();
    }

    /**
     * ボーク・カウントの消去。
     * @throws RuleException ルールの実行に失敗
     */
    public void resetBalk()
        throws RuleException
    {
        // ルール：ボーク・カウントの消去
        final RuleEngineIF rule = super.getRuleEngine("ResetBalk");

        if (rule.check()) {
            rule.run();
        }
    }

    /**
     * ボーク・カウントの加算。
     * @throws RuleException ルールの実行に失敗
     */
    public void countBalk()
        throws RuleException
    {
        // ルール：ボーク・カウントの加算
        final RuleEngineIF rule = super.getRuleEngine("CountBalk");

        if (rule.check()) {
            rule.run();
        }
    }


    /**
     * ユーザー情報の取得。
     * @throws RuleException ルールの実行に失敗
     */
    public void getUserInformation()
        throws RuleException
    {
        // ルール：ユーザー情報の取得
        final RuleEngineIF rule = super.getRuleEngine("GetUserInformation");

        if (rule.check()) {
            rule.run();
        }
    }
}
