package jp.co.hogehoge.system;

import java.sql.Date;

// commons
import jp.co.cam.usax.util.text.format.datetime.DateTimeFactory;
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.context.UserInfo;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.system.help.HELP0000SEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】活動履歴画面。
 * <br>
 * <br>Copyright(c) 2017 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class ActivityEvent
    extends AbstractExampleScreenEvent
{
    /** serialVersionUID */
    private static final long serialVersionUID = -2017L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");
                super.getRuleEngine("Initialize").run();
            }

            // ユーザー情報
            final UserInfo info = UserInfo.getUserInfo();

            if (!info.hasAuthority(new String[]{"SYSTEM_ADMIN"})) {

                if (info.hasAuthority(new String[]{"CORPORATE_ADMIN"})) {

                    super.setString("SEARCH_CORPORATE_CODE", info.getCorporateCode());

                } else {

                    super.setString("SEARCH_CORPORATE_CODE", info.getCorporateCode());
                    super.setString("SEARCH_USER_CODE",      info.getUserCode());
                    super.setString("SEARCH_USER_NAME",      info.getUserName());
                    this.btn_search_event();
                }
            }
        }
    }


    /**
     * 検索処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_search_check()
        throws RuleException
    {
        try {

            Date t = super.getDate("SEARCH_T_DATE");
    
            if (t != null) {
            // 検索条件に日付(至)が設定されている場合

                // 検索時の時間を考慮し+1日する
                t = new Date(DateTimeFactory.get().addDate(t, 0, 0, 1).getTime());
                super.setDate("SEARCH_T_NEXT", t);
            }

        } catch (final Exception wr) {
        // 日付の加算に失敗した場合(発生しない)

            super.setString("SEARCH_T_DATE", null);
            super.setString("SEARCH_T_NEXT", null);
        }
        return (super.getRuleEngine("find").check());
    }

    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        super.getRuleEngine("find").run();

        this.rewrite("一覧再描画");
    }


    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_group_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "SEARCH_GROUP_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_NAME,     "SEARCH_GROUP_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));
        e.setString("BRANCH_CODE",    super.getString("SEARCH_BRANCH_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：カレンダー。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_search_f_date_event()
        throws RuleException
    {
        // 遷移先：日付入力補助
        final HELP0000SEvent e = new HELP0000SEvent();

        e.setString(HELP0000SEvent.RES_DATE, "SEARCH_F_DATE");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：カレンダー。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_search_t_date_event()
        throws RuleException
    {
        // 遷移先：日付入力補助
        final HELP0000SEvent e = new HELP0000SEvent();

        e.setString(HELP0000SEvent.RES_DATE, "SEARCH_T_DATE");

        super.setNextPage(e);
    }
    

    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "一覧再描画":
                ctrl.addRewrite("BLOCK_ACTIVITY_LIST");
                break;
        }
    }
}
