package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.MessageIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.ScreenLockIF;
import jp.co.cam.usax.web.event.EventIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;


/**
 * 【Example】法人管理画面(詳細)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1000DEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                if (super.isCreated()) {

                    this.btn_mode_view_event();

                    if (super.getString("CORPORATE_CODE_BACK") == null) {
                    // 他者により対象の法人コードが存在しない場合の考慮

                        // メッセージ
                        final MessageIF msg = super.getMessage();

                        msg.addMessage(
                          msg.getMessageResource().get(
                            "COM8002",
                            new String[] {super.getString("CORPORATE_CODE")}
                          )
                        );

                        // 状態:保存済み → 新規データに変更
                        super._create = false;
                        this.btn_mode_edit_event();
                    }

                } else {
                    this.btn_mode_edit_event();
                }

            } else {

                log.debug("------------------------------------------------------------");
                log.debug("一覧の再読込み");
                log.debug("------------------------------------------------------------");
                if (super.getBoolean("REFRESH")) {

                    // 一覧
                    final PaginationBean list;

                    list = (PaginationBean)super.getSubList("SEARCH_RESULT");
                    if (list != null) list.refresh();
                }
            }
        }
    }


    /**
     * 照会モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_view_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();
        ctrl.setReadOnlyModeAll(ScreenLockIF.READONLY.READONLY_LABEL);
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE,           "BTN_MODE_EDIT");

        this.find();
        super.saved();
        
        this.rewrite("表示モードの書換え");
        super.setInitialize(true);
    }

    /**
     * 編集モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_edit_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();

        if (this.isCreated()) {
        // 既存データ(新規作成時は詳細データの取得は不要)

            // 詳細データの取得
            this.find();

            super.saved();

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");

        } else {

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_VIEW");
            super.setInitialize(true);
        }
        this.rewrite("表示モードの書換え");
    }


    /**
     * 支社の新規登録画面へ遷移。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_create_branch_event()
        throws RuleException
    {
        // 遷移先:支社メンテナンス画面(新規登録)
        final EventIF e = new SYS1100DEvent();

        e.setString("CORPORATE_CODE", super.getString("CORPORATE_CODE"));
        e.setString("CORPORATE_NAME", super.getString("CORPORATE_NAME"));

        super.setNextPage(e);
    }

    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ========================================================================================
        // 保存処理
        // ========================================================================================
        super.getRuleEngine("Save").run();

        // ========================================================================================
        // 画面再描画(照会)
        // ========================================================================================

        // 前画面に画面の再読込みを促す
        super.getInitialBackPage().setBoolean("REFRESH", true);

        this.btn_mode_view_event();
    }


    /**
     * 順の保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_branch_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Branch.Save").check());
    }

    /**
     * 順の保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_branch_save_event()
        throws RuleException
    {
        // ========================================================================================
        // 一覧の並替え
        // ========================================================================================

        // 一覧
        final ListBeanIF list = super.getSubList("SEARCH_RESULT");

        if (list != null) list.sort("VIEWLINE ASC, BRANCH_CODE ASC");

        // ========================================================================================
        // 保存処理
        // ========================================================================================
        super.getRuleEngine("Branch.Save").run();

        // ========================================================================================
        // 画面再描画(照会)
        // ========================================================================================
        this.rewrite("一覧再描画");
    }


    /**
     * 検索処理を実行。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    private void find()
        throws RuleException
    {
        super.getRuleEngine("Find.Detail").run();
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "表示モードの書換え":
            {
                ctrl.addRewrite("BLOCK_BUTTON_CTRL1");
                ctrl.addRewrite("BLOCK_DETAIL");
                break;
            }
            case "一覧再描画":
            {
                ctrl.clearRewrite();
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
            case "全部書換え":
            {
                ctrl.clearRewrite();
                ctrl.addRewrite("ALL");
            }
        }
    }


    /**
     * 支社一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            if (pmCmd.endsWith("$BTN_SELECTED")) {
            // 詳細画面へ遷移

                // 親イベント
                final EventIF parent = super.getParentEvent();

                // 支社詳細画面(既存データ)
                final EventIF e = new SYS1100DEvent();

                e.setString("CORPORATE_CODE", parent.getString("CORPORATE_CODE"));
                e.setString("CORPORATE_NAME", parent.getString("CORPORATE_NAME"));
                e.setString("BRANCH_CODE",    super.getString("BRANCH_CODE"));
                e.saved();

                parent.setNextPage(e);

            } else
            if (pmCmd.endsWith("$BTN_DISABLE")) {
            // 無効 → 有効

                // ================================================================================
                // 状態を更新(無効 → 有効)
                // ================================================================================

                // 親イベント
                final SYS1000DEvent parent = (SYS1000DEvent)super.getParentEvent();

                super.setString("FLAG", "0");
                super.getRuleEngine("Branch.ChangeCondition").run();

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ENABLE")) {
            // 有効 → 無効

                // ================================================================================
                // 状態を更新(有効 → 無効)
                // ================================================================================

                // 親イベント
                final SYS1000DEvent parent = (SYS1000DEvent)super.getParentEvent();

                super.setString("FLAG", "9");
                super.getRuleEngine("Branch.ChangeCondition").run();

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_DELETE")) {
            // 削除 

                // 親イベント
                final SYS1000DEvent parent = (SYS1000DEvent)super.getParentEvent();

                // ルールエンジン:指定支社の削除
                final RuleEngineIF rule = super.getRuleEngine("Branch.Delete");

                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                super.setString("FLAG", "-1");
                parent.rewrite("一覧再描画");

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
               throws RuleException
        {
            super();
        }
    }
}
