package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.web.event.EventIF;
// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】支社管理画面(検索/一覧)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1100SEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");
                super.getRuleEngine("Initialize").run();

                this.btn_search_event();

            } else {

                log.debug("------------------------------------------------------------");
                log.debug("一覧の再読込み");
                log.debug("------------------------------------------------------------");
                if (super.getBoolean("REFRESH")) {

                    // 一覧
                    final PaginationBean list;

                    list = (PaginationBean)super.getSubList("SEARCH_RESULT");
                    if (list != null) list.refresh();
                }
            }
        }
    }


    /**
     * 検索処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_search_check()
        throws RuleException
    {
        return (super.getRuleEngine("Find").check());
    }

    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        super.getRuleEngine("Find").run();

        this.rewrite("一覧再描画");
    }

    /**
     * 新規登録画面へ遷移。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_create_event()
        throws RuleException
    {
        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // 詳細画面(新規登録)
        final EventIF e = new SYS1100DEvent();

        e.setString("CORPORATE_CODE", info.getCorporateCode());
        e.setString("CORPORATE_NAME", info.getCorporateName());

        super.setNextPage(e);
    }


    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        // ----------------------------------------------------------------------------------------
        // 入力チェック
        // ----------------------------------------------------------------------------------------
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ========================================================================================
        // 事前処理
        // ========================================================================================

        // ----------------------------------------------------------------------------------------
        // 一覧の並替え
        // ----------------------------------------------------------------------------------------

        // 一覧
        final ListBeanIF list = super.getSubList("SEARCH_RESULT");

        if (list != null) list.sort("VIEWLINE ASC, BRANCH_CODE ASC");

        // ========================================================================================
        // 保存処理
        // ========================================================================================
        super.getRuleEngine("Save").run();

        // ========================================================================================
        // 事後処理
        // ========================================================================================

        // ----------------------------------------------------------------------------------------
        // 画面再描画
        // ----------------------------------------------------------------------------------------
        this.rewrite("一覧再描画");
    }


    /**
     * 入力補助呼出し：支社ID。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署ID。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_group_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "SEARCH_GROUP_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_NAME,     "SEARCH_GROUP_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));
        e.setString("BRANCH_CODE",    super.getString("SEARCH_BRANCH_CODE"));

        super.setNextPage(e);
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
        }
    }


    /**
     * 支社一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            if (pmCmd.endsWith("$BTN_SELECTED")) {
            // 詳細画面へ遷移

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                // 詳細画面(既存データ)
                final EventIF e = new SYS1100DEvent();

                e.setString("CORPORATE_CODE", info.getCorporateCode());
                e.setString("CORPORATE_NAME", info.getCorporateName());
                e.setString("BRANCH_CODE",    super.getString("BRANCH_CODE"));
                e.saved();

                super.getParentEvent().setNextPage(e);

            } else
            if (pmCmd.endsWith("$BTN_DISABLE")) {
            // 無効 → 有効

                // ================================================================================
                // 状態を更新(無効 → 有効)
                // ================================================================================

                // 親イベント
                final SYS1100SEvent parent = (SYS1100SEvent)super.getParentEvent();

                super.setString("FLAG", "0");
                super.getRuleEngine("ChangeCondition").run();

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ENABLE")) {
            // 有効 → 無効

                // ================================================================================
                // 状態を更新(有効 → 無効)
                // ================================================================================

                // 親イベント
                final SYS1100SEvent parent = (SYS1100SEvent)super.getParentEvent();

                super.setString("FLAG", "9");
                super.getRuleEngine("ChangeCondition").run();

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_DELETE")) {
            // 削除 

                // 親イベント
                final SYS1100SEvent parent = (SYS1100SEvent)super.getParentEvent();

                // ルールエンジン：指定支社の削除
                final RuleEngineIF rule = super.getRuleEngine("Branch.Delete");

                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                super.setString("FLAG", "-1");
                parent.rewrite("一覧再描画");

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }
    }
}
