package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.context.UserInfo;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.MessageIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.ScreenLockIF;
import jp.co.cam.usax.web.event.EventIF;
import jp.co.cam.usax.web.WebException;

// example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】部署管理画面(詳細)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1200DEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        if (!info.hasAuthority("SYSTEM_ADMIN")) {
        // [仕様]システム管理者以外はアクティブな法人のデータのみ参照可能

            if (!info.getCorporateCode().equals(super.getString("CORPORATE_CODE"))) {
                throw new WebException("セキュリティ上の問題が発生したため処理を中断しました");
            }
        }

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");
                super.getRuleEngine("Initialize").run();

                if (super.isCreated()) {

                    this.btn_mode_view_event();

                    if (super.getString("GROUP_CODE_BACK") == null) {
                    // 他者により対象の部署コードが存在しない場合の考慮

                        // メッセージ
                        final MessageIF msg = super.getMessage();

                        msg.addMessage(
                          msg.getMessageResource().get(
                            "COM8002",
                            new String[] {super.getString("GROUP_CODE")}
                          )
                        );

                        // 状態:保存済み → 新規データに変更
                        super._create = false;
                        this.btn_mode_edit_event();
                    }

                } else {

                    if (super.getString("BRANCH_CODE") != null) {
                        super.setBoolean("isLockBranch", true);
                    }
                    this.btn_mode_edit_event();
                }
            }
        }
    }


    /**
     * 照会モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_view_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();
        ctrl.setReadOnlyModeAll(ScreenLockIF.READONLY.READONLY_LABEL);
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE,           "BTN_MODE_EDIT");
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE,           "BTN_DELETE");

        this.find();
        super.saved();
        
        this.rewrite("表示モードの書換え");
        super.setInitialize(true);
    }

    /**
     * 編集モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_edit_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();

        if (this.isCreated()) {
        // 既存データ(新規作成時は詳細データの取得は不要)

            // 詳細データの取得
            this.find();

            super.saved();

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DELETE");

        } else {

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_VIEW");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DELETE");
            super.setInitialize(true);
        }
        this.rewrite("表示モードの書換え");
    }

    /**
     * 削除処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_delete_check()
        throws RuleException
    {
        return super.getRuleEngine("Delete").check();
    }

    /**
     * 削除処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_delete_event()
        throws RuleException
    {
        // ====================================================================================
        // 事前処理
        // ====================================================================================

        // ------------------------------------------------------------------------------------
        // ログの生成
        // ------------------------------------------------------------------------------------

        // ログ
        final StringBuilder log;

        log = new StringBuilder("[DELETE GROUP MASTER]");
        log.append(System.getProperty("line.separator"));

        log.append("Corporate Code : ").append(super.getString("COMPLETE_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append("Corporate Name : ").append(super.getString("COMPLETE_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append("Branch Code : ").append(super.getString("BRANCH_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append("Branch Name : ").append(super.getString("BRANCH_NAME"));
        log.append(System.getProperty("line.separator"));

        log.append("Group Code : ").append(super.getString("GROUP_CODE"));
        log.append(System.getProperty("line.separator"));
        log.append("Group Name : ").append(super.getString("GROUP_NAME"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // ユーザーと関連情報を削除
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.getRuleEngine("Delete").run();

        } finally {

            super.setString("LOG", null);
        }

        // ========================================================================================
        // 遷移元画面へ移動
        // ========================================================================================
        // 遷移元画面
        final EventIF e = super.getInitialBackPage();

        e.getMessage().addMessage("{@message:COMPLETED_DELETE}");

        super.setNextPage(e);
    }


    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ========================================================================================
        // 保存処理
        // ========================================================================================
        super.getRuleEngine("Save").run();

        // ========================================================================================
        // 画面再描画(照会)
        // ========================================================================================

        // 前画面に画面の再読込みを促す
        super.getInitialBackPage().setBoolean("REFRESH", true);

        this.btn_mode_view_event();
    }


    /**
     * すべてのユーザーの所属を抹消する。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_del_user_group_all_event()
        throws RuleException
    {
        // 所属一覧
        final ListBeanIF list = super.getSubList("SEARCH_RESULT");

        if (list != null) {

            for (int i = 0; i < list.getSize(); i++)
            {
                ((SubEventIF)list.get(i)).data_command("SEARCH_RESULT$" + i + "$BTN_DEL_USER_GROUP");
            }
            this.rewrite("一覧再描画");
        }
    }

    /**
     * 選択ユーザーの所属を移動する。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_move_user_group_event()
        throws RuleException
    {
        // メッセージ・コンテナ
        final MessageIF parentMsg = super.getMessage();

        // 所属一覧
        ListBeanIF list = super.getSubList("SEARCH_RESULT");

        // 選択数
        int size;

        // 選択済みに絞込み
        list = (list == null) ? null : list.findGetAll("IS_SELECTED", "1");
        size = (list == null) ? 0 : list.getSize();

        if (size == 0) {
        // ERROR:ユーザーを選択してください

            parentMsg.addError("{@attribute:ERR:USER_NOT_FOUND}");

            return false;

        } else {

            // 所属情報
            SubEventIF sub;

            // 法人ID
            String corporate;

            // 支社ID
            String branch;

            // 部署ID
            String group;

            for (int i = 0; i < size; i++)
            {
                // ページネーションのため、get(i)にてすべてのデータを取得する
                // ※getList()は表示中のページのみを取得
                sub = (SubEventIF)list.get(i);

                // ================================================================================
                // 既存所属のバックアップ
                // ================================================================================
                corporate = sub.getString("CORPORATE_CODE");
                branch    = sub.getString("BRANCH_CODE");
                group     = sub.getString("GROUp_CODE");

                // ================================================================================
                // 既存所属の登録
                // ================================================================================
                try {

                    if (UserInfo.getUserInfo().hasAuthority("SYSTEM_ADMIN")) {
                    // [仕様]法人の切替えはシステム管理者のみ可能

                        sub.setString("CORPORATE_CODE", super.getString("TO_CORPORATE_CODE"));
                    }

                    sub.setString("BRANCH_CODE",    super.getString("TO_BRANCH_CODE"));
                    sub.setString("GROUP_CODE",     super.getString("TO_GROUP_CODE"));

                    // 指定された所属を登録
                    if (!sub.data_command("SEARCH_RESULT$" + i + "$BTN_ADD_USER_GROUP")) {
                    // 所属の登録に失敗

                        // 【主なエラー理由】
                        // 指定された所属が存在しない場合
                        // 既に登録済みの場合

                        // ------------------------------------------------------------------------
                        // エラーメッセージを親クラスに移動
                        // ※ページネーション中のサブBeanのメッセージは表示中のものしか表示されないため
                        // ------------------------------------------------------------------------

                        // サブメッセージ
                        final MessageIF subMsg = sub.getMessage();

                        parentMsg.addMessage(subMsg.toArrayError());
                        parentMsg.addField("TO_BRANCH_CODE");
                        parentMsg.addField("TO_GROUP_CODE");
                        subMsg.clear();

                        // エラー発生時は中断して次へ
                        continue;
                    }

                } finally {

                    sub.setString("CORPORATE_CODE", corporate);
                    sub.setString("BRANCH_CODE",    branch);
                    sub.setString("GROUP_CODE",     group);
                }
                
                // ================================================================================
                // 既存所属の削除
                // ================================================================================
                if (!sub.data_command("SEARCH_RESULT$" + i + "$BTN_DEL_USER_GROUP")) {
                   // 処理に失敗

                    // 【主なエラー理由】
                    // 所属先がすべてなくなる場合(上部で別の所属先を登録しているため発生しない)
                    throw new RuleException("所属情報の削除に失敗");
                }
            }

            // 画面再描画
            if (!parentMsg.hasError()) this.btn_mode_view_event();
        }
        return true;
    }


    /**
     * 入力補助呼出し：支社ID。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_to_branch_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "TO_CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "TO_BRANCH_CODE");

        e.setString("CORPORATE_CODE", super.getString("TO_CORPORATE_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署ID。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_to_group_event()
        throws RuleException
    {
        // 遷移先：部署入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "TO_CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "TO_BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "TO_GROUP_CODE");

        e.setString("CORPORATE_CODE", super.getString("TO_CORPORATE_CODE"));
        e.setString("BRANCH_CODE",    super.getString("TO_BRANCH_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：支社ID。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.RES_BRANCH_CODE, "BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME, "BRANCH_NAME");

        e.setString("CORPORATE_CODE", super.getString("CORPORATE_CODE"));

        super.setNextPage(e);
    }


    /**
     * 検索処理を実行。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    private void find()
        throws RuleException
    {
        super.getRuleEngine("Find.Detail").run();
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "表示モードの書換え":
            {
                ctrl.addRewrite("BLOCK_BUTTON_CTRL1");
                ctrl.addRewrite("BLOCK_DETAIL");
                break;
            }
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
            case "全部書換え":
            {
                ctrl.clearRewrite();
                ctrl.addRewrite("ALL");
            }
        }
    }


    /**
     * 支社一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            if (pmCmd.endsWith("$BTN_SELECTED")) {
            // ユーザー詳細画面へ遷移

                // 親イベント
                final EventIF parent = super.getParentEvent();

                // 詳細画面(既存データ)
                final EventIF e = new SYS2000DEvent();

                e.setString("USER_CODE", super.getString("USER_CODE"));
                e.saved();

                parent.setNextPage(e);

            } else
            if (pmCmd.endsWith("$BTN_DEL_USER_GROUP")) {
            // 所属抹消

                // 親イベント
                final SYS1200DEvent parent = (SYS1200DEvent)super.getParentEvent();

                if (!super.getRuleEngine("User.Delete").run()) {
                    return false;
                }

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                super.setString("FLAG", "-1");
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ADD_USER_GROUP")) {
            // 所属登録

                // 親イベント
                final SYS1200DEvent parent = (SYS1200DEvent)super.getParentEvent();

                if (!super.getRuleEngine("User.Create").run()) {
                    return false;
                }

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                parent.rewrite("一覧再描画");

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }
    }
}
