package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.ScreenLockIF.READONLY;
import jp.co.cam.usax.web.event.EventIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.system.help.HELP1000SEvent;


/**
 * 【Example】権限管理画面(詳細)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1900DEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                if (super.getString("AUTH_CODE") != null) {

                    // 画面制御API
                    final ScreenControllerIF ctrl = super.getPageCtrl();
                    
                    ctrl.setReadOnlyMode(READONLY.READONLY_LABEL, "AUTH_CODE");
                    ctrl.setReadOnlyMode(READONLY.READONLY_LABEL, "AUTH_NAME");
                }
            }
        }
    }


    /**
     * 入力補助呼出し：法人。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_corporate_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1000SEvent e = new HELP1000SEvent();

        e.setString(HELP1000SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1000SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");

        super.setNextPage(e);
    }


    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ========================================================================================
        // 保存処理
        // ========================================================================================
        super.getRuleEngine("Save").run();

        // ========================================================================================
        // 画面再描画(照会)
        // ========================================================================================

        // 遷移先：遷移元画面
        final EventIF e = super.getInitialBackPage();

        // 前画面に画面の再読込みを促す
        e.setBoolean("REFRESH", true);

        setNextPage(e);
    }
}
