package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.engine.RuleEngineIF;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.web.event.EventIF;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;


/**
 * 【Example】権限管理画面(検索/一覧)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS1900SEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");
                this.btn_search_event();

            } else {

                log.debug("------------------------------------------------------------");
                log.debug("一覧の再読込み");
                log.debug("------------------------------------------------------------");
                if (super.getBoolean("REFRESH")) {

                    // 一覧
                    final PaginationBean list;

                    list = (PaginationBean)super.getSubList("SEARCH_RESULT");
                    if (list != null) list.refresh();
                }
            }
        }
    }


    /**
     * 検索処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_search_check()
        throws RuleException
    {
        return (super.getRuleEngine("Find").check());
    }

    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        super.getRuleEngine("Find").run();

        this.rewrite("一覧再描画");
    }

    /**
     * 新規登録画面へ遷移。
     */
    public void btn_create_event()
    {
        // 詳細画面(新規登録)
        final EventIF e = new SYS1900DEvent();

        super.setNextPage(e);
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
        }
    }


    /**
     * 法人一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            // 親イベント
            final SYS1900SEvent parent = (SYS1900SEvent)super.getParentEvent();

            if (pmCmd.endsWith("$BTN_ADD_CORPORATE")) {
            // 詳細画面へ遷移

                // 詳細画面(既存データ)
                final EventIF e = new SYS1900DEvent();

                e.setString("AUTH_CODE", super.getString("AUTH_CODE"));
                e.setString("AUTH_NAME", super.getString("AUTH_NAME"));

                super.getParentEvent().setNextPage(e);

            } else
            if (pmCmd.endsWith("$BTN_DELETE")) {
            // 削除 

                // ルールエンジン:指定法人の削除
                final RuleEngineIF rule = super.getRuleEngine("Delete");

                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

                // ================================================================================
                // 画面の再描画
                // ================================================================================
                super.setString("FLAG", "-1");
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$UP")) {
            // 明細の[↑]ボタン(JOINに伴うセット移動)

                // ================================================================================
                // 順の振直し
                // ================================================================================

                // 権限一覧
                final PaginationBean authList = (PaginationBean)super.getParentList();

                // 権限件数
                final int authSize = authList.getSize();

                // 行データ
                SubListBeanIF sub;

                for (int row = 0; row < authSize; row++)
                {
                    sub = authList.get(row);
                    sub.setInteger("VIEWLINE", row);
                    ((SubList1)sub).data_command("$BTN_UPDATE");
                }

                // ================================================================================
                // 順の入替え
                // ================================================================================

                // 移動元の順
                final Integer line = super.getInteger("VIEWLINE");

                // 下の行を判定
                boolean isCurrent = false;

                // 権限
                String authCode = null;

                for (int row = authSize - 1; 0 <= row; row--)
                {
                    // ----------------------------------------------------------------------------
                    // 自行の検索
                    // ----------------------------------------------------------------------------
                    if (!isCurrent) {
                        isCurrent = (authList.get(row) == this);
                        continue;
                    }
                    sub = authList.get(row);

                    // ----------------------------------------------------------------------------
                    // 入替えた行と異なる権限まで進む
                    // ----------------------------------------------------------------------------
                    if (authCode != null) {
                        if (!authCode.equals(sub.getString("AUTH_CODE"))) break;
                    }

                    // ----------------------------------------------------------------------------
                    // 上行と順を入替え
                    // ----------------------------------------------------------------------------
                    super.setInteger("VIEWLINE", sub.getInteger("VIEWLINE"));
                    this.data_command("$BTN_UPDATE");

                    sub.setInteger("VIEWLINE",   line);
                    ((SubList1)sub).data_command("$BTN_UPDATE");

                    authCode = sub.getString("AUTH_CODE");
                }

                // ================================================================================
                // 画面再表示
                // ================================================================================
                authList.refresh();

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$DW")) {
            // 明細の[↓]ボタン(JOINに伴うセット移動)

                // ================================================================================
                // 順の振直し
                // ================================================================================

                // 権限一覧
                final PaginationBean authList = (PaginationBean)super.getParentList();

                // 権限件数
                final int authSize = authList.getSize();

                // 行データ
                SubListBeanIF sub;

                for (int row = 0; row < authSize; row++)
                {
                    sub = authList.get(row);
                    sub.setInteger("VIEWLINE", row);
                    ((SubList1)sub).data_command("$BTN_UPDATE");
                }

                // ================================================================================
                // 順の入替え(下行へ移動)
                // ================================================================================

                // 移動元の順
                final Integer line = super.getInteger("VIEWLINE");

                // 下の行を判定
                boolean isCurrent = false;

                // 権限ID
                String authCode = null;

                for (int row = 0; row < authSize; row++)
                {
                    // ----------------------------------------------------------------------------
                    // 自行の検索
                    // ----------------------------------------------------------------------------
                    if (!isCurrent) {
                        isCurrent = (authList.get(row) == this);
                        authCode = this.getString("AUTH_CODE");
                        continue;
                    }
                    sub = authList.get(row);

                    // ----------------------------------------------------------------------------
                    // 自行と異なる権限まで進む
                    // ----------------------------------------------------------------------------
                    if (authCode.equals(sub.getString("AUTH_CODE"))) continue;

                    // ----------------------------------------------------------------------------
                    // 下行と順を入替え
                    // ----------------------------------------------------------------------------
                    super.setInteger("VIEWLINE", sub.getInteger("VIEWLINE"));
                    this.data_command("$BTN_UPDATE");

                    sub.setInteger("VIEWLINE", line);
                    ((SubList1)sub).data_command("$BTN_UPDATE");
                    break;
                }

                // ================================================================================
                // 画面再表示
                // ================================================================================
                authList.refresh();

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_UPDATE")) {
                // [↑][↓]ボタンの処理中に実行される処理

                // ================================================================================
                // 順を更新
                // ================================================================================

                // ルールエンジン:権限の表示順を更新
                final RuleEngineIF rule = super.getRuleEngine("Update");

                rule.run();

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }
    }
}
