package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.context.UserInfo;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.MessageIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.engine.RuleEngineIF;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.io.ScreenLockIF;
import jp.co.cam.usax.web.event.EventIF;

// example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;
import jp.co.hogehoge.system.help.HELP1000SEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】ユーザー管理画面(詳細)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS2000DEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                if (super.isCreated()) {

                    this.btn_mode_view_event();

                    if (super.getString("USER_CODE_BACK") == null) {
                    // 他者により対象のユーザーIDが存在しない場合の考慮

                        // メッセージ
                        final MessageIF msg = super.getMessage();

                        msg.addMessage(
                          msg.getMessageResource().get(
                            "COM8002",
                            new String[] {super.getString("USER_CODE")}
                          )
                        );

                        // 状態:保存済み → 新規データに変更
                        super._create = false;
                        this.btn_mode_edit_event();
                    }

                } else {

                    // ユーザー情報
                    final UserInfo info = UserInfo.getUserInfo();

                    if (!info.hasAuthority("SYSTEM_ADMIN")) {
                        super.setString("CORPORATE_CODE", info.getCorporateCode());
                        super.setString("CORPORATE_NAME", info.getCorporateName());
                    }
                    this.btn_mode_edit_event();
                }
            }
        }

        if (super.getBoolean("REFRESH")) {

            log.debug("------------------------------------------------------------");
            log.debug("一覧の再読込み");
            log.debug("------------------------------------------------------------");

            // 一覧
            final PaginationBean list;

            list = (PaginationBean)super.getSubList("GROUP_LIST");
            if (list != null) list.refresh();

            super.setBoolean("REFRESH", false);
        }
    }


    /**
     * 照会モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_view_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();
        ctrl.setReadOnlyModeAll(ScreenLockIF.READONLY.READONLY_LABEL);
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_MODE_EDIT");
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_DELETE");
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_BACK");

        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_DISABLE");
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_ENABLE");
        ctrl.setReadOnlyMode(ScreenLockIF.READONLY.NONE, "BTN_ADD_GROUP");

        this.find();
        super.saved();
        
        this.rewrite("表示モードの書換え");
        super.setInitialize(true);
    }

    /**
     * 編集モードに設定。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_mode_edit_event()
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        ctrl.clearReadOnly();

        if (this.isCreated()) {
        // 既存データ(新規作成時は詳細データの取得は不要)

            // 詳細データの取得
            this.find();

            super.saved();

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DELETE");

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DISABLE");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_ENABLE");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_ADD_GROUP");

        } else {

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_VIEW");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_MODE_EDIT");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DELETE");

            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_DISABLE");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_ENABLE");
            ctrl.setReadOnlyMode(ScreenLockIF.READONLY.READONLY_LABEL, "BTN_ADD_GROUP");

            super.setInitialize(true);
        }
        this.rewrite("表示モードの書換え");
    }


    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ====================================================================================
        // 事前処理
        // ====================================================================================

        // ------------------------------------------------------------------------------------
        // ログの生成
        // ------------------------------------------------------------------------------------

        // ログ
        final StringBuilder log;

        if (super.isCreated()) {
            log = new StringBuilder("[UPDATE USER ACCOUNT]");
        } else {
            log = new StringBuilder("[CREATE USER ACCOUNT]");
        }
        log.append(System.getProperty("line.separator"));
        log.append("User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append("Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append("Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // ユーザー情報の登録・更新
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.getRuleEngine("Save").run();

        } finally {

            super.setString("LOG", null);
        }

        // ========================================================================================
        // 画面再描画(照会)
        // ========================================================================================
        this.btn_mode_view_event();
    }

    /**
     * 無効ボタン：ユーザー・アカウントを有効化する。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_disable_event()
        throws RuleException
    {
        super.setString("FLAG", "0");

        super.getRuleEngine("ChangeCondition").run();

        this.btn_mode_view_event();
    }

    /**
     * 有効ボタン：ユーザー・アカウントを無効化する際の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_enable_check()
        throws RuleException
    {
        // [仕様]自殺はできない
        if (UserInfo.getUserInfo().getUserCode().equals(super.getString("USER_CODE"))) {

            // エラー：自分のアカウントの無効化及び削除はできません
            final String err = super.getAttributeResource().getAttribute("ERR:NO_DELETE").getName();

            super.getMessage().addError(err);

            return false;
        }
        return true;
    }

    /**
     * 有効ボタン：ユーザー・アカウントを無効化する。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_enable_event()
        throws RuleException
    {
        super.setString("FLAG", "9");

        super.getRuleEngine("ChangeCondition").run();

        this.btn_mode_view_event();
    }

    /**
     * 削除処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_delete_check()
        throws RuleException
    {
        // [仕様]自殺はできない
        if (UserInfo.getUserInfo().getUserCode().equals(super.getString("USER_CODE"))) {

            // エラー：自分のアカウントの無効化及び削除はできません
            final String err = super.getAttributeResource().getAttribute("ERR:NO_DELETE").getName();

            super.getMessage().addError(err);

            return false;
        }
        return true;
    }

    /**
     * 削除処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_delete_event()
        throws RuleException
    {
        // ====================================================================================
        // 事前処理
        // ====================================================================================

        // ------------------------------------------------------------------------------------
        // ログの生成
        // ------------------------------------------------------------------------------------

        // ログ
        final StringBuilder log;

        log = new StringBuilder("[DELETE USER ACCOUNT]");
        log.append(System.getProperty("line.separator"));
        log.append("User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append("Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append("Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // ユーザーと関連情報を削除
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.getRuleEngine("Delete").run();

        } finally {

            super.setString("LOG", null);
        }

        // ========================================================================================
        // 遷移元画面へ移動
        // ========================================================================================
        // 遷移元画面
        final EventIF e = super.getInitialBackPage();

        e.getMessage().addMessage("{@message:COMPLETED_DELETE}");

        // ユーザー一覧
        final ListBeanIF list = e.getSubList("SEARCH_RESULT");

        if (list != null) {

            for (final ListBeanIF.SubListBeanIF sub : list.getList())
            {
                if (super.getString("USER_CODE").equals(sub.getString("USER_CODE"))) {
                   // 削除済みに設定

                    sub.setString("FLAG", "-1");
                }
            }
        }

        super.setNextPage(e);
    }


    /**
     * 遷移：所属登録画面へ。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_add_group_event()
        throws RuleException
    {
        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // 遷移先：所属登録画面
        final SYS2100DEvent e = new SYS2100DEvent();

        // [仕様]システム管理者以外は自社の登録しかできない(※セキュリティ)
        if (!info.hasAuthority("SYSTEM_ADMIN")) {
            e.setString(SYS2100DEvent.PRM_CORPORATE_CODE, info.getCorporateCode());
        }
        e.setString(SYS2100DEvent.PRM_USER_CODE, super.getString("USER_CODE"));
        e.setString(SYS2100DEvent.PRM_USER_NAME, super.getString("USER_NAME"));

        super.setNextPage(e);
    }


    /**
     * 入力補助呼出し：法人。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_corporate_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1000SEvent e = new HELP1000SEvent();

        e.setString(HELP1000SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1000SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：支社の入力チェック。
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean help_branch_check()
        throws RuleException
    {
        if (super.getString("CORPORATE_CODE") == null) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addError(msg.getMessageResource().get("COM0000", new String[] {"{@attribute:CORPORATE_CODE}"}));

            return false;
        }
        return true;
    }

    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString("CORPORATE_CODE", super.getString("CORPORATE_CODE"));
        e.setString("CORPORATE_NAME", super.getString("CORPORATE_NAME"));
        e.setString("BRANCH_CODE",    null);
        e.setString("BRANCH_NAME",    null);

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME,    "BRANCH_NAME");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署の入力チェック。
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean help_group_check()
        throws RuleException
    {
        if (super.getString("CORPORATE_CODE") == null) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addError(msg.getMessageResource().get("COM0000", new String[] {"{@attribute:CORPORATE_CODE}"}));

            return false;
        }
        return true;
    }

    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_group_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString("CORPORATE_CODE", super.getString("CORPORATE_CODE"));
        e.setString("CORPORATE_NAME", super.getString("CORPORATE_NAME"));
        e.setString("BRANCH_CODE",    super.getString("BRANCH_CODE"));
        e.setString("BRANCH_NAME",    super.getString("BRANCH_NAME"));
        e.setString("GROUP_CODE",     null);
        e.setString("GROUP_NAME",     null);

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_NAME,    "BRANCH_NAME");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "GROUP_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_NAME,     "GROUP_NAME");

        super.setNextPage(e);
    }


    /**
     * 検索処理を実行。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    private void find()
        throws RuleException
    {
        super.getRuleEngine("Find.Detail").run();
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     * @throws RuleException 処理中に発生したエラーを報告
     */
    private void rewrite(
        final String pmMode
    )
        throws RuleException
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "表示モードの書換え":
            {
                ctrl.addRewrite("BLOCK_BUTTON_CTRL1");
                ctrl.addRewrite("BLOCK_BUTTON_CTRL2");
                ctrl.addRewrite("BLOCK_DETAIL");
                break;
            }
            case "一覧再描画":
            {
                super.setBoolean("REFRESH", true);
                ctrl.addRewrite("BLOCK_GROUP_LIST");
                break;
            }
            case "全部書換え":
            {
                ctrl.clearRewrite();
                ctrl.addRewrite("ALL");
            }
        }
    }


    /**
     * 所属一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            // 親イベント
            final SYS2000DEvent parent = (SYS2000DEvent)super.getParentEvent();

            if (pmCmd.endsWith("$BTN_DEL_SYSTEM_ADMIN")) {
            // システム管理者権限を削除

                if (!btn_del_system_admin()) {
                    return false;
                }
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ADD_SYSTEM_ADMIN")) {
            // システム管理者権限を設定

                if (!btn_add_system_admin()) {
                    return false;
                }
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_DEL_CORPORATE_ADMIN")) {
            // 法人管理者権限を削除

                if (!btn_del_corporate_admin()) {
                    return false;
                }
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ADD_CORPORATE_ADMIN")) {
            // 法人管理者権限を設定

                if (!btn_add_corporate_admin()) {
                    return false;
                }
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.contains("$BTN_ADD_AUTH_")) {

                // ルール：権限の登録
                final RuleEngineIF rule = super.getRuleEngine("RegistrationAnyAuthority");

                // 権限一覧
                final ListBeanIF authList = super.getSubList("AUTH_LIST");

                // 権限
                final ListBeanIF.SubListBeanIF auth;

                auth = authList.get(Integer.parseInt(parseCode(pmCmd)));

                rule.getParameter().put("AUTHORITY", auth.getString("AUTH_CODE"));

                if (!rule.run()) return false;

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.contains("$BTN_DEL_AUTH_")) {

                // ルール：権限の登録
                final RuleEngineIF rule = super.getRuleEngine("RemoveAnyAuthority");

                // 権限一覧
                final ListBeanIF authList = super.getSubList("AUTH_LIST");

                // 権限
                final ListBeanIF.SubListBeanIF auth;

                auth = authList.get(Integer.parseInt(parseCode(pmCmd)));

                rule.getParameter().put("AUTHORITY", auth.getString("AUTH_CODE"));

                if (!rule.run()) return false;

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.contains("$BTN_ADD_ROLE_")) {
            // 指定部署の権限を登録


                super.setString("GROUP_AUTH", parseCode(pmCmd));
                
                super.getRuleEngine("UpdateGroupAdmin").run();

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.contains("$BTN_DEL_ROLE_")) {
            // 指定部署の権限を削除

                super.setString("GROUP_AUTH", null);

                super.getRuleEngine("UpdateGroupAdmin").run();

                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_DELETE")) {
            // 指定の所属情報を削除

                if (!super.getRuleEngine("DeleteUserGroup").run()) {
                    return false;
                }
                parent.rewrite("一覧再描画");

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
            throws RuleException
        {
            super.getRuleEngine("Find.Authority").run();
            super.getRuleEngine("Find.Role").run();
        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }

        
        /**
         * 法人管理者権限を設定。
         * @return エラー判定(true 正常 or false 異常)
         * @throws RuleException 処理中に発生したエラーを報告
         */
        public boolean btn_add_corporate_admin()
            throws RuleException
        {
            // ルールエンジン：権限の登録
            final RuleEngineIF rule = super.getRuleEngine("registrationAuthority");

            try {

                super.setString("AUTHORITY", "CORPORATE_ADMIN");
                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

            } finally {
                super.setString("AUTHORITY", null);
            }
            return true;
        }

        /**
         * 法人管理者権限を削除。
         * @return エラー判定(true 正常 or false 異常)
         * @throws RuleException 処理中に発生したエラーを報告
         */
        protected boolean btn_del_corporate_admin()
            throws RuleException
        {
            // ルールエンジン：権限の削除
            final RuleEngineIF rule = super.getRuleEngine("removeAuthority");

            try {

                super.setString("AUTHORITY", "CORPORATE_ADMIN");
                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

            } finally {
                super.setString("AUTHORITY", null);
            }
            return true;
        }

        /**
         * システム管理者を設定。
         * @return エラー判定(true 正常 or false 異常)
         * @throws RuleException 処理中に発生したエラーを報告
         */
        protected boolean btn_add_system_admin()
            throws RuleException
        {
            // ルールエンジン：権限の削除
            final RuleEngineIF rule = super.getRuleEngine("registrationAuthority");

            try {

                super.setString("AUTHORITY", "SYSTEM_ADMIN");
                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

            } finally {
                super.setString("AUTHORITY", null);
            }
            return true;
        }

        /**
         * システム管理者を削除。
         * @return エラー判定(true 正常 or false 異常)
         * @throws RuleException 処理中に発生したエラーを報告
         */
        protected boolean btn_del_system_admin()
            throws RuleException
        {
            // ルールエンジン：権限の削除
            final RuleEngineIF rule = super.getRuleEngine("removeAuthority");

            try {

                super.setString("AUTHORITY", "SYSTEM_ADMIN");
                if (rule.check()) {
                    rule.run();
                } else {
                    return false;
                }

            } finally {
                super.setString("AUTHORITY", null);
            }
            return true;
        }

        /**
         * イベント名からコードを抽出する。
         * {イベント名}_{コード}
         * @param  pmCmd イベント名
         * @return コード
         */
        private String parseCode(final String pmCmd)
        {
            return pmCmd.substring(pmCmd.lastIndexOf("_") + 1);
        }
    }
}
