package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.bean.ListBeanIF.SubListBeanIF;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.web.event.EventIF;
// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;
import jp.co.hogehoge.system.help.HELP1000SEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】ユーザー管理画面(検索/一覧)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2017
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS2000SEvent
    extends AbstractExampleScreenEvent
{
    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                super.setString("SEARCH_CORPORATE_CODE", info.getCorporateCode());
                super.setString("SEARCH_CORPORATE_NAME", info.getCorporateName());
                
                this.btn_search_event();

            } else {

                log.debug("------------------------------------------------------------");
                log.debug("一覧の再読込み");
                log.debug("------------------------------------------------------------");

                // 一覧
                final PaginationBean list
                 = (PaginationBean)super.getSubList("SEARCH_RESULT");

                if (list != null) list.refresh();
            }
        }
    }


    /**
     * 検索処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_search_check()
        throws RuleException
    {
        return (super.getRuleEngine("Find").check());
    }

    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        super.getRuleEngine("Find").run();

        this.rewrite("一覧再描画");
    }

    /**
     * 新規登録画面へ遷移。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_create_event()
        throws RuleException
    {
        // ユーザー情報
        final UserInfo info = UserInfo.getUserInfo();

        // 詳細画面(新規登録)
        final EventIF e = new SYS2000DEvent();

        if (info.hasAuthority("SYSTEM_ADMIN")) {
            e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));
        }
        super.setNextPage(e);
    }


    /**
     * 入力補助呼出し：法人。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_corporate_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1000SEvent e = new HELP1000SEvent();

        e.setString(HELP1000SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1000SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_group_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "SEARCH_BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_NAME,    "SEARCH_BRANCH_NAME");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "SEARCH_GROUP_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_NAME,     "SEARCH_GROUP_NAME");

        e.setString("CORPORATE_CODE", super.getString("SEARCH_CORPORATE_CODE"));
        e.setString("BRANCH_CODE",    super.getString("SEARCH_BRANCH_CODE"));

        super.setNextPage(e);
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
        }
    }


    /**
     * ユーザー一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            // 親イベント
            final SYS2000SEvent parent = (SYS2000SEvent)super.getParentEvent();

            if (pmCmd.endsWith("$BTN_SELECTED")) {
            // ====================================================================================
            // 詳細画面へ遷移
            // ====================================================================================

                // ユーザー情報:選択中の法人コード
                final UserInfo info = UserInfo.getUserInfo();

                // 詳細画面(既存データ)
                final EventIF e = new SYS2000DEvent();

                e.setString("CORPORATE_CODE", info.getCorporateCode());
                e.setString("CORPORATE_NAME", info.getCorporateName());
                e.setString("USER_CODE",      super.getString("USER_CODE"));
                e.saved();

                parent.setNextPage(e);

            } else
            if (pmCmd.endsWith("$BTN_DISABLE")) {
            // ====================================================================================
            // 状態を更新(無効 → 有効)
            // ====================================================================================

                // --------------------------------------------------------------------------------
                // 同ユーザーIDの行を有効状態に設定する
                // --------------------------------------------------------------------------------

                // 自身の所属する一覧
                ListBeanIF list = super.getParentList();

                list = list.findGetAll("USER_CODE", super.getString("USER_CODE"));

                for (final SubListBeanIF sub : list.getList())
                {
                    sub.setString("FLAG", "0");
                }

                // --------------------------------------------------------------------------------
                // 状態の更新
                // --------------------------------------------------------------------------------
                super.getRuleEngine("ChangeCondition").run();

                // --------------------------------------------------------------------------------
                // 画面再描画
                // --------------------------------------------------------------------------------
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_ENABLE")) {
            // ====================================================================================
            // 有効 → 無効
            // ====================================================================================

                // [仕様]自殺はできない
                if (UserInfo.getUserInfo().getUserCode().equals(super.getString("USER_CODE"))) {

                    // エラー：自分のアカウントの無効化及び削除はできません
                    final String err = super.getAttributeResource().getAttribute("ERR:NO_DELETE").getName();

                    super.getMessage().addError(err);

                    return false;
                }

                // --------------------------------------------------------------------------------
                // 同ユーザーIDの行を無効状態に設定する
                // --------------------------------------------------------------------------------

                // 自身の所属する一覧
                ListBeanIF list = super.getParentList();

                list = list.findGetAll("USER_CODE", super.getString("USER_CODE"));

                for (final SubListBeanIF sub : list.getList())
                {
                    sub.setString("FLAG", "9");
                }

                // --------------------------------------------------------------------------------
                // 状態の更新
                // --------------------------------------------------------------------------------
                super.getRuleEngine("ChangeCondition").run();

                // --------------------------------------------------------------------------------
                // 画面再描画
                // --------------------------------------------------------------------------------
                parent.rewrite("一覧再描画");

            } else
            if (pmCmd.endsWith("$BTN_DELETE")) {
            // ====================================================================================
            // 削除 
            // ====================================================================================

                // [仕様]自殺はできない
                if (UserInfo.getUserInfo().getUserCode().equals(super.getString("USER_CODE"))) {

                    // エラー：自分のアカウントの無効化及び削除はできません
                    final String err = super.getAttributeResource().getAttribute("ERR:NO_DELETE").getName();

                    super.getMessage().addError(err);

                    return false;
                }

                // --------------------------------------------------------------------------------
                // 同ユーザーIDの行を有効状態に設定する
                // --------------------------------------------------------------------------------

                // 自身の所属する一覧
                ListBeanIF list = super.getParentList();

                list = list.findGetAll("USER_CODE", super.getString("USER_CODE"));

                for (final SubListBeanIF sub : list.getList())
                {
                    sub.setString("FLAG", "-1");
                }

                // --------------------------------------------------------------------------------
                // 状態の更新
                // --------------------------------------------------------------------------------
                   super.getRuleEngine("Delete").run();

                // --------------------------------------------------------------------------------
                // 画面再描画
                // --------------------------------------------------------------------------------
                parent.rewrite("一覧再描画");

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }
    }
}
