package jp.co.hogehoge.system;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;
import jp.co.cam.usax.context.UserInfo;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.rule.RuleException;
import jp.co.cam.usax.rule.bean.MessageIF;

// web
import jp.co.cam.usax.web.event.EventIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.system.help.HELP1000SEvent;
import jp.co.hogehoge.system.help.HELP1100SEvent;
import jp.co.hogehoge.system.help.HELP1200SEvent;


/**
 * 【Example】ユーザーの所属登録画面。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class SYS2100DEvent
    extends AbstractExampleScreenEvent
{
    // パラメータ ==================================================================

    /** パラメータ：法人コード */
    public static final String PRM_CORPORATE_CODE = "CORPORATE_CODE";

    /** パラメータ：法人名 */
    public static final String PRM_CORPORATE_NAME = "CORPORATE_NAME";

    /** パラメータ：支社コード */
    public static final String PRM_BRANCH_CODE = "BRANCH_CODE";

    /** パラメータ：支社名 */
    public static final String PRM_BRANCH_NAME = "BRANCH_NAME";

    /** パラメータ：部署コード */
    public static final String PRM_GROUP_CODE = "GROUP_CODE";

    /** パラメータ：部署名 */
    public static final String PRM_GROUP_NAME = "GROUP_NAME";

    /** パラメータ：ユーザーID */
    public static final String PRM_USER_CODE = "USER_CODE";

    /** パラメータ：ユーザー名 */
    public static final String PRM_USER_NAME = "USER_NAME";


    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                if (!info.hasAuthority("SYSTEM_ADMIN")) {
                // システム管理者以外は自社のみ登録可能

                    super.setString("CORPORATE_CODE", info.getCorporateCode());
                    super.setString("CORPORATE_NAME", info.getCorporateName());
                }

                if (super.getString(PRM_GROUP_CODE) != null) {
                    super.setBoolean("isLockGROUP", true);
                }
                if (super.getString(PRM_USER_CODE) != null) {
                    super.setBoolean("isLockUser", true);
                }
            }
        }
    }


    /**
     * 保存処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_save_check()
        throws RuleException
    {
        return (super.getRuleEngine("Save").check());
    }

    /**
     * 保存処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_save_event()
        throws RuleException
    {
        // ====================================================================================
        // 事前処理
        // ====================================================================================

        // ------------------------------------------------------------------------------------
        // ログの生成
        // ------------------------------------------------------------------------------------

        // ログ
        final StringBuilder log;

        if (super.isCreated()) {
            log = new StringBuilder("[UPDATE USER ACCOUNT]");
        } else {
            log = new StringBuilder("[CREATE USER ACCOUNT]");
        }
        log.append(System.getProperty("line.separator"));
        log.append("User Name : ").append(super.getString("USER_NAME"));
        log.append(System.getProperty("line.separator"));
        log.append("Email : ").append(super.getString("USER_EMAIL"));
        log.append(System.getProperty("line.separator"));
        log.append("Phone Num : ").append(super.getString("USER_PHONE"));
        log.append(System.getProperty("line.separator"));

        // ========================================================================================
        // ユーザー情報の登録・更新
        // ========================================================================================
        try {

            super.setString("LOG", log.toString());

            super.getRuleEngine("Save").run();

        } finally {

            super.setString("LOG", null);
        }

        // ========================================================================================
        // 前画面へ戻る
        // ========================================================================================

        // 遷移先：前画面
        final EventIF e = super.getInitialBackPage();

        e.setInitialize(true);

        super.setNextPage(e);
    }


    /**
     * 入力補助呼出し：法人。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_corporate_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1000SEvent e = new HELP1000SEvent();

        e.setString(HELP1000SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1000SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：支社の入力チェック。
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean help_branch_check()
        throws RuleException
    {
        if (super.getString("CORPORATE_CODE") == null) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addError(msg.getMessageResource().get("COM0000", new String[] {"{@attribute:CORPORATE_CODE}"}));

            return false;
        }
        return true;
    }

    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_branch_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1100SEvent e = new HELP1100SEvent();

        e.setString(HELP1100SEvent.PRM_CORPORATE_CODE, super.getString("CORPORATE_CODE"));
        e.setString(HELP1100SEvent.PRM_CORPORATE_NAME, super.getString("CORPORATE_NAME"));
        e.setString(HELP1100SEvent.PRM_BRANCH_CODE,    super.getString("BRANCH_CODE"));
        e.setString(HELP1100SEvent.PRM_BRANCH_NAME,    super.getString("BRANCH_NAME"));

        e.setString(HELP1100SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1100SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");
        e.setString(HELP1100SEvent.RES_BRANCH_CODE,    "BRANCH_CODE");
        e.setString(HELP1100SEvent.RES_BRANCH_NAME,    "BRANCH_NAME");

        super.setNextPage(e);
    }

    /**
     * 入力補助呼出し：部署の入力チェック。
     * @return true 正常 or false 異常
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean help_group_check()
        throws RuleException
    {
        if (super.getString("CORPORATE_CODE") == null) {

            // メッセージ・コンテナ
            final MessageIF msg = super.getMessage();

            msg.addError(msg.getMessageResource().get("COM0000", new String[] {"{@attribute:CORPORATE_CODE}"}));

            return false;
        }
        return true;
    }

    /**
     * 入力補助呼出し：支社。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_group_event()
        throws RuleException
    {
        // 遷移先：支社入力補助
        final HELP1200SEvent e = new HELP1200SEvent();

        e.setString(HELP1200SEvent.PRM_CORPORATE_CODE, super.getString("CORPORATE_CODE"));
        e.setString(HELP1200SEvent.PRM_CORPORATE_NAME, super.getString("CORPORATE_NAME"));
        e.setString(HELP1200SEvent.PRM_BRANCH_CODE,    super.getString("BRANCH_CODE"));
        e.setString(HELP1200SEvent.PRM_BRANCH_NAME,    super.getString("BRANCH_NAME"));
        e.setString(HELP1200SEvent.PRM_GROUP_CODE,     null);
        e.setString(HELP1200SEvent.PRM_GROUP_NAME,     null);

        e.setString(HELP1200SEvent.RES_CORPORATE_CODE, "CORPORATE_CODE");
        e.setString(HELP1200SEvent.RES_CORPORATE_NAME, "CORPORATE_NAME");
        e.setString(HELP1200SEvent.RES_BRANCH_CODE,    "BRANCH_CODE");
        e.setString(HELP1200SEvent.RES_BRANCH_NAME,    "BRANCH_NAME");
        e.setString(HELP1200SEvent.RES_GROUP_CODE,     "GROUP_CODE");
        e.setString(HELP1200SEvent.RES_GROUP_NAME,     "GROUP_NAME");

        super.setNextPage(e);
    }
}
