package jp.co.hogehoge.system.help;

// commons
import jp.co.cam.usax.debug.DebuggerFactory;
import jp.co.cam.usax.debug.DebuggerIF;

// rules
import jp.co.cam.usax.USAXAnnotations.Transaction;
import jp.co.cam.usax.context.UserInfo;
import jp.co.cam.usax.rule.bean.impl.extra.PaginationBean;
import jp.co.cam.usax.rule.bean.ListBeanIF;
import jp.co.cam.usax.rule.RuleException;

// web
import jp.co.cam.usax.web.event.io.ScreenControllerIF;
import jp.co.cam.usax.web.event.EventIF;

// Example
import jp.co.hogehoge.AbstractExampleScreenEvent;
import jp.co.hogehoge.CommonExampleSubEvent;


/**
 * 【Example】支社入力補助画面(検索/一覧)。
 * <br>
 * <br>Copyright(c) 2016-2020 Camel Engineering LLP All Rights Reserved.
 * @author Camel Engineering LLP
 * @version REVIEWED ON 2020
 * <br>######################################################################
 * <br>(変更履歴)
 * <br>REVIEWED2016 Javadoc/commentの拡充
 * <br>REVIEWED2020 実装サンプルの拡充
 * <br>######################################################################
 */
@Transaction({"default"})
public class HELP1100SEvent
    extends AbstractExampleScreenEvent
{
    // パラメータ ==================================================================

    /** パラメータ：法人コード */
    public static final String PRM_CORPORATE_CODE = "CORPORATE_CODE";

    /** パラメータ：法人名 */
    public static final String PRM_CORPORATE_NAME = "CORPORATE_NAME";

    /** パラメータ：支社コード */
    public static final String PRM_BRANCH_CODE = "BRANCH_CODE";

    /** パラメータ：支社名 */
    public static final String PRM_BRANCH_NAME = "BRANCH_NAME";

    // 返却先 ====================================================================

    /** 返却先：法人コード */
    public static final String RES_CORPORATE_CODE = "RES_CORPORATE_CODE";

    /** 返却先：法人名 */
    public static final String RES_CORPORATE_NAME = "RES_CORPORATE_NAME";

    /** 返却先：支社コード */
    public static final String RES_BRANCH_CODE = "RES_BRANCH_CODE";

    /** 返却先：支社名 */
    public static final String RES_BRANCH_NAME = "RES_BRANCH_NAME";

    /** 返却先：部署コード */
    public static final String RES_GROUP_CODE = "RES_GROUP_CODE";

    /** 返却先：部署名 */
    public static final String RES_GROUP_NAME = "RES_GROUP_NAME";

    /** SerialVersionUID */
    private static final long serialVersionUID = -2020L;


    @Override
    public void page_load()
        throws RuleException
    {
        // ログAPI
        final DebuggerIF log = DebuggerFactory.get();

        log.debug("============================================================");
        log.debug("事前処理");
        log.debug("============================================================");

        if (!super.isPostBack()) {
        // 他画面からの遷移

            if (super.isInitialize()) {
            // 初期状態

                log.debug("------------------------------------------------------------");
                log.debug("初期処理");
                log.debug("------------------------------------------------------------");

                // ユーザー情報
                final UserInfo info = UserInfo.getUserInfo();

                if (info.hasAuthority("SYSTEM_ADMIN")) {
                // [仕様]法人の検索機能はシステム管理者のみ提供
                    super.setString("SEARCH_CORPORATE_CODE", super.getString("CORPORATE_CODE"));
                }

                super.getRuleEngine("Initialize").run();

                this.btn_search_event();

            } else {

                log.debug("------------------------------------------------------------");
                log.debug("一覧の再読込み");
                log.debug("------------------------------------------------------------");
                if (super.getBoolean("REFRESH")) {

                    // 一覧
                    final PaginationBean list;

                    list = (PaginationBean)super.getSubList("SEARCH_RESULT");
                    if (list != null) list.refresh();
                }
            }
        }
    }


    /**
     * 検索処理の入力チェック。
     * @return エラー判定(true 正常 or false 異常)
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public boolean btn_search_check()
        throws RuleException
    {
        return (super.getRuleEngine("Find").check());
    }

    /**
     * 検索処理。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void btn_search_event()
        throws RuleException
    {
        super.getRuleEngine("Find").run();

        this.rewrite("一覧再描画");
    }


    /**
     * 入力補助呼出し：法人。
     * @throws RuleException 処理中に発生したエラーを報告
     */
    public void help_corporate_event()
        throws RuleException
    {
        // 遷移先：法人入力補助
        final HELP1000SEvent e = new HELP1000SEvent();

        e.setString(HELP1000SEvent.RES_CORPORATE_CODE, "SEARCH_CORPORATE_CODE");
        e.setString(HELP1000SEvent.RES_CORPORATE_NAME, "SEARCH_CORPORATE_NAME");

        super.setNextPage(e);
    }


    /**
     * 画面制御を設定。
     * @param  pmMode 制御モード
     */
    private void rewrite(
        final String pmMode
    )
    {
        // 画面制御クラス
        final ScreenControllerIF ctrl = super.getPageCtrl();

        switch (pmMode)
        {
            case "一覧再描画":
            {
                ctrl.addRewrite("BLOCK_SEARCH_RESULT");
                break;
            }
        }
    }


    /**
     * 組織一覧用のサブイベント・クラス。
     */
    public static class SubList1
        extends CommonExampleSubEvent
    {
        /** SerialVersionUID */
        private static final long serialVersionUID = -2020L;


        @Override
        public boolean data_command(final String pmCmd)
            throws RuleException
        {
            if (pmCmd.endsWith("$BTN_SELECTED")) {
            // 選択

                // 親イベント
                final EventIF parent = super.getParentEvent();

                // 遷移元
                final EventIF e = parent.getInitialBackPage();

                // 返却：法人コード
                final String RES_CORPORATE_CODE = parent.getString(HELP1100SEvent.RES_CORPORATE_CODE);

                if (RES_CORPORATE_CODE != null) {
                    e.setString(RES_CORPORATE_CODE, super.getString("CORPORATE_CODE"));
                }

                // 返却：法人名
                final String RES_CORPORATE_NAME = parent.getString(HELP1100SEvent.RES_CORPORATE_NAME);

                if (RES_CORPORATE_NAME != null) {
                    e.setString(RES_CORPORATE_NAME, super.getString("CORPORATE_NAME"));
                }

                // 返却：支社コード
                final String RES_BRANCH_CODE = parent.getString(HELP1100SEvent.RES_BRANCH_CODE);

                if (RES_BRANCH_CODE != null) {
                    e.setString(RES_BRANCH_CODE, super.getString("BRANCH_CODE"));
                }

                // 返却：支社名
                final String RES_BRANCH_NAME = parent.getString(HELP1100SEvent.RES_BRANCH_NAME);

                if (RES_BRANCH_NAME != null) {
                    e.setString(RES_BRANCH_NAME, super.getString("BRANCH_NAME"));
                }

                // 返却：部署コード
                final String RES_GROUP_CODE = parent.getString(HELP1000SEvent.RES_GROUP_CODE);

                if (RES_GROUP_CODE != null) {
                    e.setString(RES_GROUP_CODE, null);
                }

                // 返却：部署名
                final String RES_GROUP_NAME = parent.getString(HELP1000SEvent.RES_GROUP_NAME);

                if (RES_GROUP_NAME != null) {
                    e.setString(RES_GROUP_NAME, null);
                }

                e.updated();

                parent.setNextPage(e);

            } else {

                return super.data_command(pmCmd);
            }
            return true;
        }


        @Override
        public void data_load()
        {

        }


        /**
         * コンストラクタ。
         * @param  pmList 親イベント
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1(
            final ListBeanIF pmList
        )
            throws RuleException
        {
            super(pmList);
        }

        /**
         * コンストラクタ。
         * @throws RuleException 処理中に発生した例外を報告
         */
        public SubList1()
            throws RuleException
        {
            super();
        }
    }
}
