/**
 * [example]共通的な機能を提供するJavascript。
 * 本JSは、Universal Frameworkシリーズと、その系統にてお使い頂けます。
 * ※利用に関するライセンスにつきましてはFramework本体に沿います。
 * ※本Scriptは参考プログラムのためサポート対象外です。
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2020
 * @see ScreenDebugger
 */
var Commons = {

    // コンテキスト・ルート
    _root : '/',

    _loading : 0,
    _checked : 0,
    _wait : ''
};


/**
 * 指定URLの「~/」をベースURLに変換。
 * @return 変換結果
 */
Commons.conversionContextRoot = function(pmURL)
{
    return pmURL.replace('~/', Commons.getContextRoot());
};

/**
 * ベースURLを設定。
 * ※画面表示時に設定してください
 * @param  pmRoot コンテキスト・ルート
 */
Commons.setContextRoot = function(pmURL)
{
    Commons.println('SET BASE-URL', pmURL);

    Commons._root = pmURL;
};

/**
 * ベースURLを取得。
 * @return ベースURL
 */
Commons.getContextRoot = function()
{
    return Commons._root;
};


/**
 * Objectを取得。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj Object名
 * @return Object
 */
Commons.getLayerObject = function(pmWin, pmObj)
{
    // ============================================================================================
    // 指定ウインドウのドキュメントを取得
    // ============================================================================================

    // ドキュメント
    var win = eval(pmWin + '.document');

    // ============================================================================================
    // 指定オブジェクトを取得
    // ============================================================================================
    if (win) {

        // 項目ID(名前を指定された場合はIDに変換)
        var id = pmObj.replace(/\$/g, '-');

        // 戻り値
        var ret;

        if (win.getElementById) {
        // 標準

            ret = win.getElementById(id);

            if (ret) return ret;

            // 救済処置：大文字統一
            ret = win.getElementById(id.toUpperCase());

            if (ret) return ret;

            // 救済処置：小文字統一
            ret = win.getElementById(id.toLowerCase());

            if (ret) return ret;

        } else
        if (win.all) {
        // 旧IE

            ret = win.all(id);

            if (ret) return ret;

            // 救済処置：大文字統一
            ret = win.all(id.toUpperCase());

            if (ret) return ret;

            // 救済処置：小文字統一
            ret = win.all(id.toLowerCase());

        } else
        if (win.layers) {
        // Netscape

            ret = win.layers[id];

            if (ret) return ret;

            // 救済処置：大文字統一
            ret = win.layers[id.toUpperCase()];

            if (ret) return ret;

            // 救済処置：小文字統一
            ret = win.layers[id.toLowerCase()];
        }
    }
};


/**
 * Styleを取得。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 * @return Style
 */
Commons.getLayerStyle = function(pmWin, pmObj)
{
    // ============================================================================================
    // 指定オブジェクトを取得
    // ============================================================================================

    // オブジェクト
    var obj = Commons.getLayerObject(pmWin, pmObj);

    // ============================================================================================
    // Styleを取得
    // ============================================================================================
    if (obj) {

        if (document.getElementById || document.all) {
        // 標準(旧IE含む)

            if (obj.style) {
                return obj.style;
            }

        } else
        if (document.layers) {
        // Netscape

            return obj;
        }
    }
};

/**
 * CSSを変更。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 * @param  pmCss Class名
 */
Commons.setStyleName = function(pmWin, pmObj, pmCss)
{
    // ============================================================================================
    // 指定オブジェクトを取得
    // ============================================================================================

    // オブジェクト
    var obj = Commons.getLayerObject(pmWin, pmObj);

    // ============================================================================================
    // Class属性を変更
    // ============================================================================================
    if (obj) {
        obj.className = pmCss;
    }
};

/**
 * Background設定
 * @param pmID  ID
 * @param pmRGB 色
 */
Commons.setBgColor = function(pmID, pmRGB)
{
    // ============================================================================================
    // 指定オブジェクトを取得
    // ============================================================================================

    // オブジェクト
    var obj = Commons.getLayerStyle('self', pmID);

    // ============================================================================================
    // BackgroundColor属性を変更
    // ============================================================================================
    if (obj) {

        obj.backgroundColor = pmRGB;

    } else {
        throw 'Object not found - ' + pmID;
    }
};

/**
 * ReadOnly設定
 * @param pmID  項目ID
 * @param pmVal true ON or false OFF
 */
Commons.setReadOnly = function(pmID, pmVal)
{
    // ============================================================================================
    // 指定オブジェクトを取得
    // ============================================================================================

    // オブジェクト
    var obj = Commons.getLayerObject('self', pmID);

    // ============================================================================================
    // ReadOnly属性を変更
    // ============================================================================================
    if (obj) {

        // ----------------------------------------------------------------------------------------
        // 背景色の切替え
        // ----------------------------------------------------------------------------------------
        if (pmVal) {
        // 読取り専用(薄灰色)

            Commons.setBgColor('self', pmID, '#DDDDDD');

        } else {

            Commons.setBgColor('self', pmID, '');
        }

        // ----------------------------------------------------------------------------------------
        // 読取り専用モードのON/OFF
        // ----------------------------------------------------------------------------------------
        obj.readOnly = pmVal;

    } else {
        throw 'Object not found - ' + pmID;
    }
};


/**
 * Display設定
 * @param pmID  ID
 * @param pmVal 設定(inline / block / none)
 */
Commons.setDisplay = function(pmID, pmVal)
{
    // ============================================================================================
    // 指定オブジェクトのStyleを取得
    // ============================================================================================

    // スタイル
    var style = Commons.getLayerStyle('self', pmID);

    // ============================================================================================
    // Display属性を変更
    // ============================================================================================
    if (style) {

        Commons.println(pmID + '.style.display', pmVal);

        style.display = pmVal;

    } else {
        throw 'Object not found - ' + pmID;
    }
};


/**
 * 指定されたログをコンソールに出力します。
 * ※consoleに対応しないブラウザは動作しません。
 * @param  pmLog 出力するログ
 * @param  pmOpt オプション
 */
Commons.println = function(pmLog, pmOpt)
{
    try {

        if (ScreenDebugger.isDebugMode()) {
        // [仕様]デバッグモード時のみ書換え内容を出力する

            if (pmOpt) {
                console.log(pmLog + " - " + pmOpt);
            } else {
                console.log(pmLog);
            }
        }

    } catch (wr) {
    // consoleに未対応の場合

        // 処理なし
    }
};


/**
 * Headタグの読込み。
 * ※Headタグの置換えに伴うScriptの再読込みを行う
 * @param  pmOld 旧タグ
 * @param  pmNew 新タグ
 */
Commons.changeHeadTag = function(pmOld, pmNew)
{
    // ============================================================================================
    // 旧Headタグのコンテンツをクリア
    // ============================================================================================
    pmOld.text = '';
    pmOld.textContent = '';
    pmOld.innerHTML = '';

    // ============================================================================================
    // 子タグの解析
    // ============================================================================================

    // 子タグ一覧
    var list = pmNew.childNodes;

    for (var i = 0; i < list.length; i++)
    {
        if (list[i].tagName) {
        // タグの場合

            if (list[i].tagName.toUpperCase() == 'SCRIPT') {
            // [仕様]scriptは生成し直さなければ読込みされない

                if (list[i].src) {
                // 外部参照

                    // 基本モジュールは読込み済のため置換え不要
                    if ((list[i].src.indexOf('commons.js') != -1)
                     || (list[i].src.indexOf('request.js') != -1)
                     || (list[i].src.indexOf('usax.js') != -1)) {
                        continue;
                    }

                    // scriptタグ
                    var script = document.createElement('script');

                    script.onloadstart = function()
                    {
                        Commons._loading++;
                    };
                    script.onloadend = function()
                    {
                        Commons._loading--;
                    };
                    script.src = list[i].src;

                 } else {

                     // scriptタグ
                     var script = document.createElement('script');
                     script.innerHTML = list[i].innerHTML;
                 }

                 pmOld.appendChild(script);
                 continue;
            }
            pmOld.appendChild(list[i]);
        }
    }
};

/**
 * 指定されたタグを置換え。
 * @param  pmOld 旧タグ
 * @param  pmNew 新タグ
 */
Commons.replaceTag = function(pmOld, pmNew)
{
    if (pmOld.tagName.toUpperCase() == 'HEAD') {
    // ============================================================================================
    // Headタグの置換え
    // ============================================================================================

        Commons.changeHeadTag(pmOld, pmNew);

    } else {
    // ============================================================================================
    // タグの置換え
    // ============================================================================================

        Commons.println(pmNew.innerHTML);

        pmOld.parentNode.replaceChild(pmNew, pmOld);
    }
};

/**
 * 指定IDのオブジェクトをHTMLパーツの内容で書換える。
 * @param  pmHTML HTMLパーツ
 * @param  pmID ID
 */
Commons.changeTagById = function(pmHTML, pmID)
{
    // ============================================================================================
    // 置換え先のタグを取得
    // ============================================================================================

    // 置換え対象
    var old = Commons.getLayerObject('self', pmID);

    // ============================================================================================
    // 置換え
    // ============================================================================================
    if (old) {

        // HTMLデータに含まれるタグを取得
        var dat = Commons.getTag(pmHTML, old.tagName, pmID);

        if (dat) {
        // Tagあり(置換え)

            Commons.replaceTag(old, dat);

        } else {
        // Tagなし(非表示)

            Commons.setDisplay(pmID, 'none');
        }

    } else {
        throw 'Object not found - ' + pmID;
    }
};

/**
 * 指定IDのオブジェクトをHTMLパーツの内容で書換える。
 * @param  pmHTML HTMLパーツ
 * @param  pmTag タグ名
 */
Commons.changeTag = function(pmHTML, pmTag)
{
    // ============================================================================================
    // 置換え先のタグを取得
    // ============================================================================================

    // 置換え対象
    var old = document.getElementsByTagName(pmTag);

    // ============================================================================================
    // 置換え
    // ============================================================================================
    if (old && old.length != 0) {

        // HTMLデータに含まれるタグを取得
        var dat = Commons.getTag(pmHTML, pmTag);

        if (dat) {
        // Tagあり(置換え)

            Commons.replaceTag(old[0], dat);

        } else {
        // Tagなし(非表示)

            Commons.setDisplay(pmID, 'none');
        }

    } else {
        throw 'Object not found - ' + pmTag;
    }
};


/**
 * HTMLパーツの指定されたIDのタグを取得。
 * @param  pmHTML HTML
 * @param  pmTag タグ
 * @param  pmID ID
 * @return Tag
 */
Commons.getTag = function(pmHTML, pmTag, pmID)
{
    // 戻り値
    var ret;

    // ============================================================================================
    // HTMLパーツをキャンパスに展開
    // ============================================================================================

    // 解析用のキャンパス
    var dummy = document.createElement('html');

    dummy.innerHTML = pmHTML;

    // ============================================================================================
    // タグ名から捜索
    // ============================================================================================

    // 該当するタグ一覧
    var tag;
    
    if (pmTag) {
    // タグ名の指定がある場合
        tag = dummy.getElementsByTagName(pmTag);
    } else {
    // タグ名の指定がない場合(全タグから探す※非推奨)
        tag = dummy.getElementsByTagName("*");
    }

    if (tag) {

        for (var i = 0; i < tag.length; i++)
        {
            if (pmID && tag[i].id) {
                if (pmID.toUpperCase() == tag[i].id.toUpperCase()) return tag[i];
            } else {
                return tag[i];
            }
        }
    }
    return;
};



/**
 * FORMオブジェクトの一覧を取得。
 * 【対象となるタグ】
 *  ・button
 *  ・input
 *  ・select
 *  ・textarea
 * @param  pmWin ウインドウ(フレーム)名
 * @return FORMオブジェクトの一覧
 */
Commons.getFormObjects = function(pmWin)
{
    // 対象のdocument
    var win = eval(pmWin + ".document");

    // 戻り値
    var ret = [];

    // 件数
    var len = 0;

    for (var i = 0; i < win.forms[0].elements.length; i++)
    {
        temp = win.forms[0].elements[i];

        name = temp.tagName.toUpperCase();

        if ((name == 'INPUT')
         || (name == 'BUTTON')
         || (name == 'SELECT')
         || (name == 'TEXTAREA')) {
            ret[len++] = temp;
        }
    }
    return ret;
};

/**
 * 指定されたFORMオブジェクトのを取得。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 * @return FORMオブジェクト
 */
Commons.getFormObject = function(pmWin, pmObj)
{
    // ============================================================================================
    // 項目の取得
    // ============================================================================================

    // IDを元に取得(大文字・小文字を見ない)
    var ret = Commons.getLayerObject(pmWin, pmObj);

    if (ret) {
    // --------------------------------------------------------------------------------------------
    // 単一項目
    // --------------------------------------------------------------------------------------------

        return ret;

    } else {
    // --------------------------------------------------------------------------------------------
    // 選択項目
    // --------------------------------------------------------------------------------------------

        // 選択肢一覧
        var tmp = [];

        for (var i = 0; true; i++)
        {
            // 選択肢
            var selection = Commons.getLayerObject(pmWin, pmObj + "-" + i + "-SELECTED");

            if (selection) {

                tmp[tmp.length] = selection;

            } else {
                break;
            }
        }
        if (tmp.length != 0) ret = tmp;
    }
    return ret;
};

/**
 * FORMオブジェクトの値を取得。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 * @param  pmVal 初期値
 * @return 値
 */
Commons.getFormValue = function(pmWin, pmObj, pmVal)
{
    // ============================================================================================
    // 値の取得
    // ============================================================================================

    // 値の設定取得元
    var obj = Commons.getFormObject(pmWin, pmObj);

    if (obj) {

        if (obj.length) {
        // ----------------------------------------------------------------------------------------
        // 選択項目(checkbox/radio)
        // ----------------------------------------------------------------------------------------

            // 選択済み一覧
            var tmp = '';

            for (var i = 0; i < obj.length; i++)
                
            {
                if (obj.checked) {
                // 選択済みの場合

                    if (tmp) {
                        tmp += ',';
                    }
                    tmp += obj.value;
                }
            }

            if (tmp) return tmp;

        } else {
        // ----------------------------------------------------------------------------------------
        // 単一項目
        // ----------------------------------------------------------------------------------------

            if (obj.value) return obj.value;
        }
    }

    // ============================================================================================
    // 初期値の返却
    // ============================================================================================
    if (pmVal) {
        return pmVal;
    } else {
        return '';
    }
};

/**
 * FORMオブジェクトに値を設定。
 * ※値設定後はonchangeイベントが実行されます。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 * @param  pmVal 値
 */
Commons.setFormValue = function(pmWin, pmObj, pmVal)
{
    // ============================================================================================
    // 値の設定
    // ============================================================================================

    // 値の設定設定先
    var obj = Commons.getFormObject(pmWin, pmObj);

    if (obj) {

        if (obj.length) {
        // ----------------------------------------------------------------------------------------
        // 選択項目(checkbox/radio)
        // ----------------------------------------------------------------------------------------

            for (var i = 0; i < obj.length; i++)
            {
                if (obj.value == pmVal) {

                    if (!obj.checked) {
                       // 未選択

                        obj.checked = true;

                        if (obj.onchange) obj.onchange();
                    }
                    break;
                }
            }

        } else {
        // ----------------------------------------------------------------------------------------
        // 単一項目
        // ----------------------------------------------------------------------------------------

            if (obj.value != pmVal) {
            // 値が異なる

                obj.value = pmVal;

                if (obj.onchange) obj.onchange();
            }
        }
    }
};

/**
 * FORMオブジェクトの値をクリア。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名一覧
 */
Commons.clearFormValue = function(pmWin, pmObj)
{
    if (pmObj) {

        var base = eval(pmWin + '.document');

        var obje;

        for (var i1 = 0; i1 < pmObj.length; i1++)
        {
            obje = base.getElementsByName(pmObj[i1]);

            if (Array.isArray(obje) || (obje.item !== void 0 && obje.length !== void 0)) {
            // 配列

                for (var i2 = 0; i2 < obje.length; i2++)
                {
                    if (obje[i2].tagName.toUpperCase() === 'TEXTAREA') {
                        obje[i2].value = '';
                    } else
                    if (obje[i2].tagName.toUpperCase() === 'SELECT') {
                        obje[i2].selectedIndex = 0;
                    } else
                    if (obje[i2].type) {

                        if (obje[i2].type.toUpperCase() === 'CHECKBOX') {
                            obje[i2].checked = false;
                        } else
                        if (obje[i2].type.toUpperCase() === 'RADIO') {
                            obje[i2].checked = (i2 === 0);
                        } else {
                            obje[i2].value = '';
                        }
                    }
                }
            }
        }
    }
};


/**
 * FORMオブジェクトのボタン/リンクを押下。
 * @param  pmWin ウインドウ(フレーム)名
 * @param  pmObj オブジェクト名
 */
Commons.clickButton = function(pmWin, pmObj)
{
    // ドキュメント・オブジェクト
    var obj = Commons.getLayerObject(pmWin, pmObj);

    if (obj && obj.click) obj.click();
};


/**
 * ローディング後に、指定Functionを実行。
 */
Commons.checkLoading = function(pmFunc)
{
    if (pmFunc) Commons._wait = pmFunc;

    if (0 < Commons._loading && Commons._checked < 999) {
    // ローディング中

        Commons.println('Loading... ' + Commons._checked);

        // 10ms後に再チェック
        setTimeout(Commons.checkLoading, 10);
        Commons._checked++;

    } else {

        Commons._loading = 0;
        Commons._checked = 0;

        if (Commons._wait) {

            Commons.println('call function - ' + pmFunc);

            setTimeout(Commons._wait, 10);
            Commons._wait = '';
        }
    }
};


/**
 * カーソルを変更する。
 * @param  pmType タイプ
 */
Commons.changeCursor = function(pmType)
{
    try {

        document.body.style.cursor = pmType;

    } catch(wr) {
    // 未対応のブラウザ

        // 処理なし
    }
};

/**
 * 先頭項目をフォーカスする。
 */
Commons.firstFocus = function()
{
    if (document.forms[0]) {

        for (var i = 0; i < document.forms[0].elements.length; i++)
        {
            // [仕様]最初にインデックスが1で設定されている項目を対象とする
            if (document.forms[0].elements[i].tabIndex == '1') {
                document.forms[0].elements[i].focus();
                break;
            }
        }
    }
};

/**
 * 次の項目をフォーカスする。
 * @param  pmObj 現在のフォーカス・オブジェクト
 */
Commons.nextFocus = function(pmObj)
{
    // 今のインデックス
    var indx = '';

    // 次のインデックス
    var next;

    // フォーカス候補
    var min = pmObj;

    // フラグ：カレント以降
    var flg = false;

    // ========================================================================
    // 現在のインデックスを取得
    // ========================================================================
    if (pmObj.tabIndex) {
        indx = eval(pmObj.tabIndex);
    }
    if (!(indx)) return;

    // ========================================================================
    // 次の項目を検索
    // ========================================================================

    // 先頭位置のフォーカス
    for (var i = 0; i < document.forms[0].elements.length; i++)
    {
        // [仕様]非活性の項目は対象外
        if (document.forms[0].elements[i].disabled || document.forms[0].elements[i].readOnly) continue;

        // [仕様]インデックスが設定されている項目のみ対象とする
        if (document.forms[0].elements[i].tabIndex) {

            next = eval(document.forms[0].elements[i].tabIndex);

            if (next <= 0 || next < indx) continue;

            if (indx == next) {
            // 同インデックス

                // ------------------------------------------------------------
                // カレント・オブジェクト以降の同インデックスを発見
                // ------------------------------------------------------------
                if (flg) {
                    min = document.forms[0].elements[i];
                    break;
                }

                // ------------------------------------------------------------
                // カレント・オブジェクトを検索
                // ------------------------------------------------------------
                if (document.forms[0].elements[i].name) {
                    if (document.forms[0].elements[i].name == pmObj.name) {
                        flg = true;
                        continue;
                    }
                }
                if (document.forms[0].elements[i].id) {
                    if (document.forms[0].elements[i].id == pmObj.id) {
                        flg = true;
                        continue;
                    }
                }

            } else
            if (indx < next) {
            // 次のフォーカス候補

                // ------------------------------------------------------------
                // カレント・オブジェクト以降の最小インデックスを検索
                // ------------------------------------------------------------
                if (min) {

                    if (next < eval(min.tabIndex)) {
                        min = document.forms[0].elements[i];
                    }

                } else {
                    min = document.forms[0].elements[i];
                }
            }

        } else {
            continue;
        }
    }

    // ========================================================================
    // フォーカスの設定
    // ========================================================================
    if (min) {

        if (min.tagName.toUpperCase() == 'BUTTON') {
        // [仕様]ボタンの場合はクリックする

            min.click();

        } else {
            min.focus();
        }
    }
};


/**
 * 画面トップへスクロールする。
 */
Commons.scrollToTop = function()
{
    var scrolled = ( window.pageYOffset !== undefined ) ? window.pageYOffset: document.documentElement.scrollTop;
    window.scrollTo( 0, Math.floor( scrolled / 2 ) );
    if ( scrolled > 0 ) {
      window.setTimeout( Commons.scrollToTop, 30 );
    }
};
