/**
 * [example]入力補助機能を提供するJavascript。
 * 本JSは、Universal Frameworkシリーズと、その系統にてお使い頂けます。
 * ※ライセンスに関しましてはFramework本体に沿います。
 * ※参考プログラムのためサポート対象外です。
 *
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2016
 * @see commons.js
 * @see request.js
 */
var InputHelp = {

    // ウインドウ
    _inputHelp : {}
};


/**
 * 日付入力補助画面を開く。
 * <br>法人コードを指定することで休業日を色分けして表示します。
 * @param  pmCorporate 法人コード
 * @param  pmCalendar 種別
 * @param  pmDate 日付
 * @param  reDate 返却先:日付
 * @param  reWeek 返却先:曜日
 * @param  reY    返却先:年
 * @param  reM    返却先:月
 * @param  reD    返却先:日
 */
InputHelp.callCalendar = function(pmCorporate, pmCalendar, pmDate, reDate, reWeek, reY, reM, reD)
{
    // 入力補助画面：日付入力補助画面(カレンダー)
    var path = '~/system/help/HELP0000S.jsp';

    // パラメータ
    var parm = '?SEARCH_CORPORATE_CODE={@PM_CORPORATE}'
             + '&SEARCH_CALENDAR_CODE={@PM_CALENDAR}'
             + '&DATE={@PM_DATE}'
             + '&RE_DATE={@DATE}'
             + '&RE_WEEK={@WEEK}'
             + '&RE_Y={@YYYY}'
             + '&RE_M={@MM}'
             + '&RE_D={@DD}'
             + '&USAX_FRAME_NAME=HELP_CALENDAR';

    parm = InputHelp.encodePath(parm, '{@PM_CORPORATE}', pmCorporate)
    parm = InputHelp.encodePath(parm, '{@PM_CALENDAR}', pmCalendar)
    parm = InputHelp.encodePath(parm, '{@PM_DATE}', pmDate)
    parm = InputHelp.encodePath(parm, '{@DATE}', reDate)
    parm = InputHelp.encodePath(parm, '{@WEEK}', reWeek)
    parm = InputHelp.encodePath(parm, '{@YYYY}', reY)
    parm = InputHelp.encodePath(parm, '{@MM}',   reM)
    parm = InputHelp.encodePath(parm, '{@DD}',   reD)

    window.open(Commons.conversionContextRoot(path + parm), 'HELP_CALENDAR', 'width=860,height=380,menubar=no,location=no,resizable=no,scrollbars=no,status=no');
}

InputHelp.encodePath = function(pmPath, pmKey, pmVal)
{
    if (pmVal) {
        return pmPath.replace(pmKey, encodeURI(pmVal));
    } else {
        return pmPath.replace(pmKey, '');
    }
}


/**
 * 指定エレメントのレイヤーベースIDを取得する。
 * (例)SubList1$0$Sample → 解:SubList1―0
 * @param  pmObje 対象のエレメント
 * @return レイヤーベースID
 */
InputHelp.getLayerBaseID = function(pmObje)
{
    // 戻り値：レイヤーベースID
    var ret = '';

    // 位置情報
    var idx;

    if (pmObje) {

        if (pmObje.name) {

            ret = pmObje.name;

            idx = ret.lastIndexOf('$');

            if (0 < idx) {

                ret = ret.substring(0, idx);
                ret = ret.replace(/\$/g, '-');

            } else {
                ret = '';
            }

        } else {

            ret = pmObje.id;

            idx = ret.lastIndexOf('$');

            if (0 < idx) {
                ret = ret.substring(0, ret.lastIndexOf('-'));
            } else {
                ret = '';
            }
        }
    }
    return ret;
};


/**
 * 指定項目の初期値を設定する。
 * @param  pmInit 返却先一覧
 * @param  pmFind 検索項目一覧
 */
InputHelp.setInitialValues = function(pmInit, pmFind)
{
    if (window.opener) {
        if (window.opener.closed) self.close();
    } else {
        self.close();
    }

    // 返却先の項目名
    var name;

    // 検索条件
    var find;

    for (var i = 0; i < pmInit.length; i++)
    {
        name = Commons.getLayerObject('self', pmInit[i]).value;

        if (name && pmFind[i]) {

            find = Commons.getLayerObject('self', pmFind[i]);

            if (find.value != window.opener.document.getElementById(name).value) {

                find.value = window.opener.document.getElementById(name).value;

                if (find.onchange) find.onchange();
            }
        }
    }
};

/**
 * 指定項目の値を返却する。
 * @param  pmRess 返却先一覧
 * @param  pmData 返却データ一覧
 */
InputHelp.setValues = function(pmRess, pmData)
{
    if (window.opener) {
        if (window.opener.closed) self.close();
    } else {
        self.close();
    }

    // 返却先の項目名
    var name;

    for (var i = 0; i < pmRess.length; i++)
    {
        name = Commons.getLayerObject('self', pmRess[i]).value;

        if (name) {

            if (window.opener.document.getElementById(name).value != pmData[i]) {

                window.opener.document.getElementById(name).value = pmData[i];

                if (window.opener.document.getElementById(name).onchange) {
                    window.opener.document.getElementById(name).onchange();
                }
            }
        }
    }
    self.close();
};
