// ############################################################################
// [USAX](example)通信機能を提供するJavascript。
//
// 本JSは、Universal Frameworkシリーズと、その系統にてお使い頂けます。
// ※ライセンスに関しましてはFramework本体に沿います。
// ※参考プログラムのためサポート対象外です。
//
// @author Camel Engineering LLP J.Matsuo
// @varsion REVIEW ON 2018
//
// 2016 新規作成
// 2018 サブ・ウインドウのオープン機能を追加
//
// @see upload.js HTML5 分割アップロードJS
// ############################################################################

/** COOKIEの保存期限(日) */
var EXPIRY_DATE_COOKIES = 365 * 1;

// HTTP通信
var HTTPRequest = {

    // サブ・ウインドウ
    _windows : new Array(),

    // 画面ロック情報
    _locked : new Array(),
    _lock   : false,
    _try    :   0,
    _max    : 999
};


/**
 * 指定画面を別ウインドウにて開く。
 * @param  pmName ウインドウ名
 * @param  pmPath URL
 * @param  pmOpt  オプション
 * @return ウインドウ
 */
HTTPRequest.openWindow = function(pmName, pmPath, pmOpt)
{
    // --------------------------------------------------------------------------------------------
    // 既存ウインドウを片す
    // --------------------------------------------------------------------------------------------
    if (HTTPRequest._windows[pmName]) {
    // 既存ウインドウあり

        // 既存ウインドウが開いている場合は閉じる
        if (!HTTPRequest._windows[pmName].closed) {
            if (HTTPRequest._windows[pmName].close) HTTPRequest._windows[pmName].close();
        }

        delete HTTPRequest._windows[pmName];
    }

    // --------------------------------------------------------------------------------------------
    // 新しいウインドウを開く
    // --------------------------------------------------------------------------------------------
    var win = window.open(Commons.conversionContextRoot(pmPath), pmName, pmOpt);

    try {
    
        // 最大5秒待機
        var LIMIT_WAIT = new Date().getTime() + 5 * 1000;

        while (new Date().getTime() < LIMIT_WAIT)
        {
            if (win && win.close && win.document && win.document.body && win.document.getElementById) break;
        }
        win.focus();

    } catch (ex) {
        try { console.log(ex); } catch (wr) { }
    }

    HTTPRequest._windows[pmName] = win;

    return win;
};


/**
 * 指定のURLに対してイベントを送信する。
 * @param  pmPath 送信先URL
 * @param  pmParm パラメータ
 * @param  pmBack onreadystatechange
 */
HTTPRequest.connect = function(pmPath, pmParm, pmBack)
{
    // ============================================================================================
    // リクエスト送信準備
    // ============================================================================================
    // HttpRequest
    var request = HTTPRequest.createHttpRequest();

    // 送信データ
    var data = HTTPRequest.createRequestValues();

    for (var key in pmParm)
    {
        if (pmParm.hasOwnProperty(key)) {

            if (key.toUpperCase() == 'USAX_RESPONSE_TYPE') {
            // レスポンス型の切替え
                request.responseType = pmParm[key];
            } else {
                data.append(key, pmParm[key]);
            }
        }
    }

    // ============================================================================================
    // リクエストの送信
    // ============================================================================================
    // ※loadendはsafari(5.1.7)では動かず
    request.onreadystatechange = pmBack;

    request.open("POST", pmPath, true);

    request.setRequestHeader("Access-Control-Allow-Origin", location.origin);

    for (var header in this.headers)
    {
        request.setRequestHeader(header, headers[header]);
    }


    if (HTTPRequest.getCookie("USAX_ID")) {
        request.setRequestHeader("X-CSRFToken", HTTPRequest.getCookie("USAX_ID"));
    }
    request.withCredentials = true;

    if (document.characterset) {
        document.characterset = 'UTF-8';
    } else
    if (document.charset) {
        document.charset = 'UTF-8';
    }

    request.send(data);
};


/**
 * 入力フォームのデータを格納する。
 */
HTTPRequest.createRequestValues = function()
{
    // 戻り値
    var ret = new FormData();

    // ============================================================================================
    // 入力項目を取集しエンコード
    // ============================================================================================

    // 入力項目
    var temp;

    // 識別ID
    var name;

    // タイプ
    var type;

    for (var i = 0; i < document.forms[0].elements.length; i++)
    {
        temp = document.forms[0].elements[i];
        name = temp.tagName.toUpperCase();

        if (name == 'INPUT') {

            type = temp.type.toUpperCase();

            if (type == 'TEXT'
             || type == 'HIDDEN'
             || type == 'NUMBER'
             || type == 'TEL'
             || type == 'URL'
             || type == 'EMAIL'
             || type == 'PASSWORD') {

                ret.append(temp.name, encodeURIComponent(temp.value));

            } else
            if (type == 'CHECKBOX') {

                if (temp.checked) {
                    ret.append(temp.name, encodeURIComponent(temp.value));
                } else {
                    ret.append(temp.name, "");
                }

            } else
            if (type == 'RADIO') {

                if (temp.checked) {
                    ret.append(temp.name, encodeURIComponent(temp.value));
                }

            } else
            if (type == 'FILE') {

                // 項目名:PATH送信Field
                var KEY_PATH = temp.name + '_PATH';

                if (!document.getElementById(KEY_PATH)) {
                // ファイル分割送信の対象でない場合

                    if (temp.files[0]) {
                        ret.append(KEY_PATH,  temp.files[0].name);
                        ret.append(temp.name, temp.files[0]);
                    } else {
                        ret.append(KEY_PATH,  '');
                        ret.append(temp.name, '');
                    }
                }
            }

        } else
        if (name == 'SELECT') {
               ret.append(temp.name, encodeURIComponent(temp.value));
        } else
        if (name == 'TEXTAREA') {
            ret.append(temp.name, encodeURIComponent(temp.value));
        }
    }
    return ret;
};


/**
 * HttpRequestの生成。
 * @return HttpRequest
 */
HTTPRequest.createHttpRequest = function()
{
    if (window.XMLHttpRequest) {
        try { return new XMLHttpRequest(); } catch(wr) { ;}
    }

    if (window.ActiveXObject) {

        try { return new ActiveXObject("Msxml2.XMLHTTP.6.0"); } catch(wr) { ;}
        try { return new ActiveXObject("Msxml2.XMLHTTP.3.0"); } catch(wr) { ;}
        try { return new ActiveXObject("Msxml2.XMLHTTP");     } catch(wr) { ;}

        try { return new ActiveXObject("Microsoft.XMLHTTP"); } catch(wr) { ;}
    }
    return;
};


/**
 * リクエストの処理状況を監視する。
 */
HTTPRequest.monitoring = function()
{
    if (HTTPRequest.isLocked()) {
    // ロック中

        if (HTTPRequest._try < HTTPRequest._max) {
        // 監視中(一定時間後に再確認)

            setTimeout('HTTPRequest.monitoring()', 100);
            HTTPRequest._try++;

        } else {
        // ロック解除

            HTTPRequest.unlock();
        }
    }
};

/**
 * ロック状態を判定する。
 * @return true ロック中 or false 未ロック
 */
HTTPRequest.isLocked = function()
{
    return HTTPRequest._lock;
};

/**
 * ロックを解除する。
 */
HTTPRequest.unlock = function()
{
    if (HTTPRequest.isLocked()) {

        // --------------------------------------------------------------------------------------------
        // ロック中のオブジェクトを解放
        // --------------------------------------------------------------------------------------------
        for (var i = 0; i < HTTPRequest._locked.length; i++)
        {
            btn = HTTPRequest._locked[i];

            if (btn.id) {
                sty = Commons.getLayerStyle('self', btn.id);
            } else {
                sty = btn.style;
            }

            if (btn.tagName.toUpperCase() == 'BUTTON') {
            // ボタンを解除

                btn.disabled = false;

                if (sty) {
                    sty.opacity = 1;
                }

            } else
            if (btn.tagName.toUpperCase() == 'A') {
            // アンカーを解除

                if (sty) {
                    sty.pointerEvents = 'auto';
                    sty.opacity = 1;
                }
            }
        }

        // ----------------------------------------------------------------------------------------
        // ロックを解除
        // ----------------------------------------------------------------------------------------
        HTTPRequest._locked.length = 0;
        HTTPRequest._lock = false;
        HTTPRequest._try = 0;
    }
};

/**
 * ロックを開始する。
 */
HTTPRequest.lock = function()
{
    if (!HTTPRequest.isLocked()) {

        // ----------------------------------------------------------------------------------------
        // ロックを開始
        // ----------------------------------------------------------------------------------------
        HTTPRequest._lock = true;
        HTTPRequest._try = 0;

        // ----------------------------------------------------------------------------------------
        // ボタンをロック
        // ----------------------------------------------------------------------------------------
    
        // ボタン／アンカー
        var btn;

        // スタイル
        var sty;

        if (btn = document.getElementsByTagName('button')) {

            // スタイル
            var sty;

            for (var i = 0; i < btn.length; i++)
            {
                if (btn[i].id) {
                    sty = Commons.getLayerStyle('self', btn[i].id);
                } else {
                    sty = btn[i].style;
                }

                if (sty) {

                    // 元々無効なボタンは対象外
                    if (btn[i].disabled) continue;

                    btn[i].disabled = true;
                    sty.opacity = 0.5;

                    HTTPRequest._locked.push(btn[i]);
                }
            }
        }

        // ----------------------------------------------------------------------------------------
        // アンカーをロック
        // ----------------------------------------------------------------------------------------
        if (btn = document.getElementsByTagName('a')) {

            for (var i = 0; i < btn.length; i++)
            {
                if (btn[i].id) {
                    sty = Commons.getLayerStyle('self', btn[i].id);
                } else {
                    sty = btn[i].style;
                }
                if (sty) {

                    // 元々無効なボタンは対象外
                    if (sty.pointerEvents == 'none') continue;

                    sty.pointerEvents = 'none';
                    sty.opacity = 0.5;

                    HTTPRequest._locked.push(btn[i]);
                }
            }
        }

        // ----------------------------------------------------------------------------------------
        // 常駐監視の開始
        // ----------------------------------------------------------------------------------------
        HTTPRequest.monitoring();
    }
};


/**
 * Cookieの利用が可能か判定。
 */
HTTPRequest.isUsedCookie = function()
{
    return HTTPRequest.getCookie('USING_COOKIE');
};

/**
 * Cookieの取得。
 * @param  キー名
 * @return データ
 */
HTTPRequest.getCookie = function(pmName)
{
    if (document.cookie) {

        // Cookie一覧
        var cookies = document.cookie.split(';');

        for (var i = 0; i < cookies.length; i++)
        {
            // Cookie
            var cookie = cookies[i].replace(/^\s+|\s+$/g, '');

            if (cookie.substring(0, pmName.length + 1) == (pmName + '=')) {

                return decodeURIComponent(cookie.substring(pmName.length + 1));
            }
        }
    }
    return;
};

/**
 * Cookieの保存。
 * @param  キー名
 * @param  データ
 */
HTTPRequest.setCookie = function(pmName, pmData)
{
    document.cookie = pmName + '=' + encodeURIComponent(pmData) + ';max-age=' + (60 * 60 * 24 * EXPIRY_DATE_COOKIES);
};
HTTPRequest.setCookie('USING_COOKIE', 'true');
