/**
 * [example]Canvasタグにて画像を出力するためのJavascript。
 * 独自Frameworkを開発する際の参考にご利用ください。
 *
 *  (使い方)
 *  ★HTMLにてcssとjsを読込み
 *  <canvas id="canvas" width="800" height="400"></canvas>
 *
 *  ★JSにてdebugを実行
 *  Canvas.setCanvas(getLayerObject('self', 'myCanvas'));
 *
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2016
 */

var Canvas = {

  canvas : '',
  ctx    : '',

  color  : '#666666',
  alpha  : 1.0,
  size   : 3,

  height : 400,
  width  : 800,

  /* 一時データ */
  _drag : false,
  _x :'',
  _y :''
};

/**
 * キャンバスの設定。
 * @param  pmObje キャンバス
 */
Canvas.setCanvas = function(pmObje)
{
    Canvas.canvas = pmObje;

    if (pmObje) {

        if (pmObje.height) {
            Canvas.height = pmObje.height;
        }
        if (pmObje.width) {
            Canvas.width = pmObje.width;
        }

        Canvas.ctx = pmObje.getContext('2d');
        Canvas.clear();

    } else {
        Canvas.ctx = '';
    }

    // マウス・イベント
    Canvas.canvas.addEventListener('mousedown', Canvas.onClick, false);
    Canvas.canvas.addEventListener('mousemove', Canvas.onDrag,  false);
    Canvas.canvas.addEventListener('mouseout',  Canvas.onDrop,  false);
    Canvas.canvas.addEventListener('mouseup',   Canvas.onDrop,  false);

    Canvas.canvas.addEventListener('touchstart', Canvas.onClick, false);
    Canvas.canvas.addEventListener('touchmove', Canvas.onDrag,  false);
    Canvas.canvas.addEventListener('touchend',  Canvas.onDrop,  false);

    can.addEventListener("mousedown",onMouseDown,false);
    can.addEventListener("mousemove",onMouseMove,false);
    can.addEventListener("mouseup",onMouseUp,false);};

/**
 * ペンサイズの設定。
 * @param  pmSize 数値
 */
Canvas.setPenSize = function(pmSize)
{
    Canvas.size = pmSize;
};

/**
 * ペン色の設定。
 * @param  pmConf RGB
 */
Canvas.setColor = function(pmConf)
{
    Canvas.color = pmConf;
};

/**
 * 透過の設定。
 * @param  pmConf 透過率
 */
Canvas.setAlpha = function(pmConf)
{
    Canvas.alpha = pmConf / 10;
};

/**
 * クリア処理。
 */
Canvas.clear = function()
{
    if (Canvas.ctx) {

        Canvas.ctx.beginPath();
        Canvas.ctx.fillStyle   = '#F5F5F5';
        Canvas.ctx.globalAlpha = 1.0;

        Canvas.ctx.fillRect(0, 0, Canvas.width, Canvas.height);

        Canvas.onDrop();
    }
};


/**
 * クリック処理。
 * @param  e イベント
 */
Canvas.onClick = function(e)
{
    if (e.button === 0) {

        Canvas._drag = true;

        var rect = e.target.getBoundingClientRect();
        var X = ~~(e.clientX - rect.left);
        var Y = ~~(e.clientY - rect.top);

        Canvas.draw(X, Y);
    }
};


/**
 * ドラッグ処理。
 * @param  e イベント
 */
Canvas.onDrag = function(e)
{
    if (Canvas._drag) {

        var rect = e.target.getBoundingClientRect();
        var X = ~~(e.clientX - rect.left);
        var Y = ~~(e.clientY - rect.top);

        Canvas.draw(X, Y);
    }
};


/**
 * 描画終了。
 * (ドロップ／領域から外れる)
 */
Canvas.onDrop = function()
{
    Canvas._drag = false;
    Canvas._x = '';
    Canvas._y = '';
};


/**
 * 描画処理。
 * @param  X 横軸
 * @param  Y 縦軸
 */
Canvas.draw = function(X, Y)
{
    if (Canvas.ctx) {

        Canvas.ctx.beginPath();
        Canvas.ctx.globalAlpha = Canvas.alpha;

        if (Canvas._x === '') {
            Canvas.ctx.moveTo(X, Y);
        } else {
            Canvas.ctx.moveTo(Canvas._x, Canvas._y);
        }

        Canvas.ctx.lineTo(X, Y);

        Canvas.ctx.lineCap     = 'round';
        Canvas.ctx.lineWidth   = Canvas.size * 2;
        Canvas.ctx.strokeStyle = Canvas.color;
        Canvas.ctx.stroke();
    }

    Canvas._x = X;
    Canvas._y = Y;
};


Canvas.rendering = function()
{
    return Canvas.canvas.toDataURL('image/png');
};
