/**
 * [example]画面Beanの保持データをデバッグ出力するためのJavascript。
 * 独自Frameworkを開発する際の参考にご利用ください。
 *
 *  (使い方)
 *  ★HTMLにてcssとjsを読込み
 *  <link type="text/css" rel="stylesheet" href="~/_style/tool/debug.css" />
 *  <script src="~/_script/tool/debug.js"></script>
 *
 *  ★JSにてdebugを実行
 *  ScreenDebugger.debug();
 *
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2016
 * @see commons.js
 * @see request.js
 * @see usax.js
 */
var ScreenDebugger = {

    // デバッグ出力の表示状況（true 表示 or false 非表示）
    _visibleScreenDebug : false,

    // デバッグ・コントロール
    _isDebug : false
};

/**
 * [public]デバッグ・コントロールを設定。
 * @param  pmFlag 真偽(true デバッグ・モード or false 標準)
 */
ScreenDebugger.setDebugMode = function(pmFlag)
{
    ScreenDebugger._isDebug = pmFlag;
};


/**
 * [public]デバッグ・コントロールを判定。
 * @return 判定結果(true デバッグ・モード or false 標準)
 */
ScreenDebugger.isDebugMode = function()
{
    return ScreenDebugger._isDebug;
};


/**
 * [public]デバッグ出力を実行。
 */
ScreenDebugger.debug = function()
{
    if (ScreenDebugger._visibleScreenDebug) {
    // 既に開いている場合は閉じる

        ScreenDebugger.closeDebug();
        return;
    }

    if (!ScreenDebugger.isDebugMode()) {
        return;
    }

    // 送信データ
    var formData = new FormData();

    // リクエスト
    var xhr = HTTPRequest.createHttpRequest();

    if (xhr) {

        xhr.onreadystatechange = function(e)
        {
            if (this.readyState == 4) {

                if (this.status == 200) {
                    ScreenDebugger.debugWrite(xhr.responseText);
                }
                delete xhr;
            }
        };

        xhr.open("POST", USAX.getLocationURL(), true);

        formData.append('USAX_EVENT_EXEC', 'SCREEN DEBUG');
        xhr.send(formData);

    } else {
        throw 'Sory, unsupported your browser.';
    }
};


/**
 * [private]ログ(HTML)を出力する。
 * @param  pmHTML レスポンスHTML
 */
ScreenDebugger.debugWrite = function(pmHTML)
{
    Commons.changeTagById(pmHTML, 'SCREEN_DEBUG');
    Commons.setDisplay('SCREEN_DEBUG',        'block');
    Commons.setBgColor('BTN_SCREEN_DEBUGGER', '#AA0000');

    Commons.getLayerObject('self', 'SCREEN_DEBUG').focus();
    ScreenDebugger._visibleScreenDebug = true;
};

/**
 * [private]デバッグを終了する。
 */
ScreenDebugger.closeDebug = function()
{
    Commons.setBgColor('BTN_SCREEN_DEBUGGER', '');
    Commons.setDisplay('SCREEN_DEBUG', 'none');

    ScreenDebugger._visibleScreenDebug = false;
};
