/**
 * [example]ファイル分割アップロードするためのJavascript。
 * 独自Frameworkを開発する際の参考にご利用ください。
 *
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2016
 * @see commons.js
 * @see request.js
 * @see usax.js
 */
var ChankFileUploader = {

    // アップロード上限サイズ
    limitFileSize : 2000000000,

    // 分割サイズ
    chankSize : 20480,

    // ファイル情報
    _fieldCode : '',
    _fieldName : '',
    _fileName  : '',
    _fileType  : '',
    _fileSize  :  0,
    _file      : '',

    // 分割回数
    _chankTotal : 0,

    // 送信回数
    _chankCount : 0,

    // 読込み済サイズ
    _read : 0,

    // 処理後実行Script
    _onClear : '',
    _onLoad  : ''
};


//読込み中の画像
var IMAGE_LOADING = '~/_style/images/loading.gif';


/**
 * ファイルの分割アップロードに対応しているか判定する。
 * @param  pmFile ファイル項目
 * @return 判定結果(true:対応 / false:未対応)
 */
ChankFileUploader.isSupportedBrowser = function(pmFile)
{
    if (window.FormData) {
        ; // FormDataに対応するブラウザ
    } else {
        return false;
    }

    // File APIの対応確認
    if (window.File && window.FileReader && window.FileList && window.Blob) {
        ; // File APIに対応するブラウザ
    } else {
        return false;
    }

    // sliceの対応確認
    if (pmFile) {

        if (pmFile.files) {

            if (0 < pmFile.files.length) {
                return (pmFile.files[0].slice || pmFile.files[0].webkitSlice || pmFile.files[0].mozSlice);
            }
        }
    }
    return true;
};

/**
 * アップロード上限サイズを設定。
 * @param  pmSize 上限サイズ
 */
ChankFileUploader.setLimitFileSize = function(pmSize)
{
    ChankFileUploader.limitFileSize = pmSize;
};

/**
 * 分割サイズを設定。
 * @param  pmSize 分割サイズ
 */
ChankFileUploader.setChankSize = function(pmSize)
{
    ChankFileUploader.chankSize = pmSize;
};

/**
 * File項目の参照ボタンのStyleを取得。
 * @param  pmFile File項目ID
 * @return 参照ボタン[Style]
 */
ChankFileUploader.getFileSelectorStyle = function(pmFile)
{
    // 参照ボタンのIDは「File項目ID＋_SELECT」とする
    return Commons.getLayerStyle('self', pmFile + '_SELECT');
};

/**
 * File項目の参照ボタンを取得。
 * @param  pmFile File項目ID
 * @return 参照ボタン
 */
ChankFileUploader.getFileSelector = function(pmFile)
{
    // 参照ボタンのIDは「File項目ID＋_SELECT」とする
    return Commons.getLayerObject('self', pmFile + '_SELECT');
};


/**
 * アップロード・ステータスを設定。
 * @param  pmFile File項目ID
 * @param  pmFlag true 開始 or false 終了
 */
function setFileLoading(pmFile, pmFlag)
{
    // File参照ボタンのStyle
    var style = ChankFileUploader.getFileSelectorStyle(pmFile);

    if (style) {

        // 参照ボタン
        var btn = ChankFileUploader.getFileSelector(pmFile);

        if (pmFlag) {

            style.backgroundImage = 'url("' + Commons.conversionContextRoot(IMAGE_LOADING) + '")';
            if (btn) btn.disabled = true;

        } else {

            style.backgroundImage = '';
            style.display = 'none';
            if (btn) btn.disabled = false;
        }
    }
}

/**
 * 対象ファイルを設定。
 * @param  pmFile File項目ID
 * @param  pmPath Fileパス
 * @param  pmExec 処理後の実行Script
 */
ChankFileUploader.setFileName = function(pmFile, pmPath, pmExec)
{
    // ファイル・パス
    var path = pmPath.replace(/\\/g, "/");

    // パス[]
    var addr = path.split('/');

    Commons.setFormValue('self', pmFile + '_PATH', addr[addr.length - 1]);

    var file = Commons.getLayerObject('self', pmFile);

    if (ChankFileUploader.isSupportedBrowser(file)) {

        ChankFileUploader._onLoad = pmExec;

        ChankFileUploader.send(file);
    }
};

/**
 * アップロード・ステータスを判定。
 * @param  pmFile File項目ID
 * @return 判定結果(true 読込み中 or false 未読込み／読込み済)
 */
ChankFileUploader.isLoading = function(pmFile)
{
    // File参照ボタンのStyle
    var style = ChankFileUploader.getFileSelectorStyle(pmFile);

    if (style) {
        return (style.backgroundImage.indexOf(Commons.conversionContextRoot(IMAGE_LOADING)) != -1);
    }
};


/**
 * File項目の値を消去。
 * @param  pmFile File項目ID
 * @param  pmExec 処理後の実行Script
 */
ChankFileUploader.clear = function(pmFile, pmExec)
{
    // ドキュメント・オブジェクト
    var obj = Commons.getLayerObject('self', pmFile);

    if (obj) {

        HTTPRequest.lock();

        // ====================================================================
        // アップロード・ステータスを消去する
        // ====================================================================

        // File参照ボタンのStyle
        var style = ChankFileUploader.getFileSelectorStyle(pmFile);
        if (style) {
            style.backgroundImage = '';
            style.display = 'inline';
        }

        // 参照ボタン
        var btn = ChankFileUploader.getFileSelector(pmFile);
        if (btn)  btn.disabled = false;

        // ====================================================================
        // File項目をクローンして置き換えることで値を消す
        // ====================================================================

        // クローン
        var tmp = obj.cloneNode(true);

        Commons.replaceTag(obj, tmp);

        // ====================================================================
        // 自画面にPOST形式にて接続
        // ====================================================================

        // request
        var xhr = HTTPRequest.createHttpRequest();

        xhr.onreadystatechange = function(e)
        {
            if (this.readyState == 4) {

                if (this.status == 200) {
                // 正常処理

                    try {

                        // 処理結果
                        var html = this.responseText;

                        Commons.changeTagById(html, pmFile + '_SELECT');
                        Commons.changeTagById(html, pmFile + '_CLEAR');
                        Commons.changeTagById(html, pmFile + '_PATH');
                        Commons.changeTagById(html, pmFile);

                        // メッセージ・ブロックを書換え
                        Commons.changeTagById(this.responseText, MESSAGE);

                        // 処理後実行Script
                        if (ChankFileUploader._onClear) {

                            try {

                                eval(ChankFileUploader._onClear);

                            } finally {
                                ChankFileUploader._onClear = '';
                            }
                        }

                    } catch (ex) {
                        throw 'レスポンスの解析に失敗 - (' + xhr.status + ')' + ex.message;
                    } finally {
                        HTTPRequest.unlock();
                    }
                }
            }
        };

        xhr.open("POST", USAX.getLocationURL(), true);

        xhr.setRequestHeader("Access-Control-Allow-Origin", location.origin);

        if (HTTPRequest.getCookie("USAX_ID")) {
            xhr.setRequestHeader("X-CSRFToken", HTTPRequest.getCookie("USAX_ID"));
        }
        xhr.withCredentials = true;

        // --------------------------------------------------------------------
        // ヘッダーの設定
        // --------------------------------------------------------------------
        for (var header in this.headers)
        {
            xhr.setRequestHeader(header, headers[header]);
        }
        xhr.setRequestHeader("index", ChankFileUploader._chankCount);

        // --------------------------------------------------------------------
        // データの設定
        // --------------------------------------------------------------------

        // FormData
        var formData = new FormData();

        formData.append("USAX_TEMPORARY_FIELD",  Commons.getLayerObject('self', pmFile).name);
        formData.append("USAX_TEMPORARY_NAME",   '');
        formData.append("USAX_TEMPORARY_FILE",   '');
        formData.append("USAX_TEMPORARY_APPEND", '');

        formData.append("USAX_FRAME_NAME", Commons.getFormValue('self', 'USAX_FRAME_NAME'));
        formData.append("USAX_EVENT_EXEC", "REMOVEDATA");

        // --------------------------------------------------------------------
        // 送信
        // --------------------------------------------------------------------
        ChankFileUploader._onClear = pmExec;
        xhr.send(formData);
    }
};


/**
 * 送信処理。
 * @param  pmFile 入力項目(File)
 */
ChankFileUploader.send = function(pmFile)
{
    // File APIの対応確認
    if (!ChankFileUploader.isSupportedBrowser()) {
        throw 'Sory, unsupported your browser.';
    }

    // Fileの指定を確認
    if (pmFile) {
        if (pmFile.files.length == 0) return;
    } else {
        return;
    }

    // ファイルサイズの確認
    if (ChankFileUploader.limitFileSize < pmFile.size) {
        throw 'limit over - size = ' + pmFile.size + ' / limit = ' + ChankFileUploader.limitFileSize;
    }

    // ファイル情報の取得
    ChankFileUploader._fieldCode = pmFile.id;
    ChankFileUploader._fieldName = (pmFile.name) ? pmFile.name : pmFile.id;
    ChankFileUploader._file      = pmFile.files[0];
    ChankFileUploader._fileName  = ChankFileUploader._file.name;
    ChankFileUploader._fileSize  = ChankFileUploader._file.size;
    ChankFileUploader._fileType  = ChankFileUploader._file.type;

    // 分割回数の初期化
    ChankFileUploader._chankTotal = Math.ceil(ChankFileUploader._fileSize / ChankFileUploader.chankSize);
    ChankFileUploader._chankCount = 0;
    ChankFileUploader._read = 0;

    setFileLoading(ChankFileUploader._fieldCode, true);

    // 送信開始
    ChankFileUploader.next();
};

/**
 * [private]分割された１件の処理を実行。
 */
ChankFileUploader.next = function()
{
    // ========================================================================
    // 送信範囲の算出
    // ========================================================================

    // 送信範囲の終了位置
    var top = ChankFileUploader._read;
    var end = ChankFileUploader._read + ChankFileUploader.chankSize;

    if (ChankFileUploader._fileSize < end) {
        end = ChankFileUploader._fileSize;
    }

    // ========================================================================
    // ファイル分割
    // ========================================================================

    // Blob
    var blob = ChankFileUploader._file;

    if (blob.slice) {
        blob = blob.slice(top, end);
    } else
    if (ChankFileUploader._file.webkitSlice) {
        blob = blob.webkitSlice(top, end);
    } else
    if (ChankFileUploader._file.mozSlice) {
        blob = blob.mozSlice(top, end);
    }

    // ========================================================================
    // 分割ファイルの読込み
    // ========================================================================

    // FileReader API
    var fileReader = new FileReader();

    // 読込み後のコールバックを定義
    fileReader.onloadend = function(e)
    {
        if (e.target.readyState == FileReader.DONE) {
            ChankFileUploader.post(new Uint8Array(e.target.result), end);
        }
    };

    // 読込み
    if (blob) {
        fileReader.readAsArrayBuffer(blob);
    }
};

/**
 * 分割された１件を送信。
 * @param  pmData binary
 * @param  pmRead 読込み済みサイズ
 */
ChankFileUploader.post = function(pmData, pmRead)
{
    // 終了判定
    var eof = (ChankFileUploader._fileSize <= pmRead);

    ChankFileUploader._read = pmRead;

    HTTPRequest.lock();

    // ========================================================================
    // 自画面にPOST形式にて接続
    // ========================================================================

    // request
    var xhr = HTTPRequest.createHttpRequest();

    xhr.open("POST", USAX.getLocationURL(), true);

    xhr.setRequestHeader("Access-Control-Allow-Origin", location.origin);

    if (HTTPRequest.getCookie("USAX_ID")) {
        xhr.setRequestHeader("X-CSRFToken", HTTPRequest.getCookie("USAX_ID"));
    }
    xhr.withCredentials = true;

    // ========================================================================
    // 送信後のコールバックを定義
    // ========================================================================
    xhr.onreadystatechange = function()
    {
        if (this.readyState === 4) {

            if (this.status == 200) {

                if (eof) {
                // 終了

                    // --------------------------------------------------------
                    // 送信済みFileをクリアする
                    // --------------------------------------------------------
                    try {

                        // 入力項目
                        var obj = Commons.getLayerObject('self', ChankFileUploader._fieldCode);

                        Commons.replaceTag(obj, obj.cloneNode(true));

                        setFileLoading(ChankFileUploader._fieldCode, false);

                        // メッセージ・ブロックを書換え
                        Commons.changeTagById(this.responseText, MESSAGE);

                        // 処理後実行Script
                        if (ChankFileUploader._onLoad) {

                            try {

                                if (HTTPRequest.isLocked()) HTTPRequest.unlock();

                                eval(ChankFileUploader._onLoad);

                            } finally {
                                ChankFileUploader._onLoad = '';
                            }
                        }

                    } finally {
                        HTTPRequest.unlock();
                    }
                    return;
                }

                ChankFileUploader._chankCount++;
                ChankFileUploader.next();
            }
        }
    };

    // ========================================================================
    // ヘッダーの設定
    // ========================================================================
    for (var header in this.headers)
    {
        xhr.setRequestHeader(header, headers[header]);
    }
    xhr.setRequestHeader("index", ChankFileUploader._chankCount);

    // ========================================================================
    // データの設定
    // ========================================================================

    // FormData
    var formData = new FormData();

    formData.append("USAX_TEMPORARY_FIELD",  ChankFileUploader._fieldName);
    formData.append("USAX_TEMPORARY_NAME",   ChankFileUploader._fileName);
    formData.append("USAX_TEMPORARY_FILE",   new Blob([pmData], {type:"application/octet-stream"}));
    formData.append("USAX_TEMPORARY_APPEND", (eof) ? '0' : '1');

    formData.append("USAX_FRAME_NAME", Commons.getFormValue('self', 'USAX_FRAME_NAME'));
    formData.append("USAX_EVENT_EXEC", "RESUMEDATA");

    // ========================================================================
    // 送信
    // ========================================================================
    xhr.send(formData);
};
