/**
 * [USAX](example)Servletにイベント実行リクエストを送るスクリプト。
 * 独自Frameworkを開発する際の参考にご利用ください。
 *
 * @author Camel Engineering LLP J.Matsuo
 * @varsion REVIEW ON 2016
 * @see commons.js
 * @see request.js
 */
var USAX = {

};


// 画面先頭のフォーカス位置のオブジェクトID
var PAGETOP = 'BLOCK_CONTENTS';

// ブロックID:メッセージ
var MESSAGE = 'USAX_MESSAGE';

// 色：フォーカス時の項目背景色
var COLOR_FOCUS = '#D0F0D0';

//色：エラー時の項目背景色
var COLOR_ERROR = '#FED0D0';


/**
 * ブラウザの対応状況を判定する。
 * @return 判定結果（true 対応 or false 未対応）
 */
USAX.isSupportedBrowser = function()
{
    try {

        if (!(HTTPRequest.createHttpRequest())) {
            throw 'XMLHttpRequest - object not found.';
        }

        if (!(window.FormData)) {
            throw 'FormData - object not found.';
        }

// ※ファイル分割UPLOADは全体の機能には影響なし
//      // File APIの対応確認
//      if (!(window.File && window.FileReader && window.FileList && window.Blob)) {
//          alert(window.File);
//          alert(window.FileReader);
//          alert(window.FileList);
//          alert(window.Blob);
//          throw 'Un supported - File API.';
//      }

    } catch (ex) {
        return false;
    }
    return true;
};


/**
 * 現在表示中の画面のパスを取得。
 * @return 現在の画面URL
 */
USAX.getLocationURL = function()
{
    // 現在の画面URL
    var ret = Commons.getFormValue('self', 'USAX_EVENT_PAGE', '');

    if (ret) {
        return Commons.conversionContextRoot('~' + ret);
    } else {
        return location.pathname;
    }
};

/**
 * 現在表示中の画面を履歴に追加。
 */
USAX.pushState = function()
{
    // 画面URL
    var path = USAX.getLocationURL();

    try {

        if (path != location.pathname) {
        // 二重登録の防止を考慮

            // URL書換え
            history.pushState('usax_state', '', path);
        }

    } catch (ex) {
        try { console.log(ex); } catch (wr) { }
    }
};


/**
 * メニューの表示・非表示を切替。
 */
USAX.menu = function()
{
    // 画面全体のブロック
    var win = Commons.getLayerStyle('self', 'BLOCK_MENU');

    if (win) {

        if (win.display == '' || win.display == 'none') {
            win.display = 'block';
        } else {
            win.display = 'none';
        }
    }
};

/**
 * 画面の先頭にフォーカスする。
 * @param  pmObj 入力項目
 */
USAX.gotoScreenTop = function()
{
    // 画面全体のブロック
    var win = Commons.getLayerObject('self', PAGETOP);

    if (win) win.focus();
};


// ############################################################################
// メッセージの表示制御
// ############################################################################

/**
 * メッセージ対象のフィールド色を変更する。
 * @param  pmRGB 色
 */
USAX.changeMessageTargetColor = function(pmRGB)
{
    // 対象フィールド・データ
    var keys = Commons.getLayerObject('self', MESSAGE + '_FIELD');

    if (keys) {

        // 対象フィールド一覧
        var list = eval(keys.innerHTML);

        // 項目
        var obj;

        for (var i = 0 ; i < list.length; i++)
        {
            if (obj = Commons.getLayerObject('self', list[i])) {
                Commons.setBgColor(obj.id, pmRGB);
            }
        }

    } else {

        throw '[実装の問題/環境の問題]返却情報「' + MESSAGE + '_FIELD」が見つかりません(通信状況および障害有無を確認してください)';
    }
};

/**
 * メッセージ対象のフィールドか判定する。
 * @param  pmKey 項目ID
 * @return 判定結果 true 対象 or false その他
 */
USAX.isMessageTargetColor = function(pmKey)
{
    // 対象フィールド・データ
    var keys = Commons.getLayerObject('self', MESSAGE + '_FIELD');

    if (keys) {

        // 対象フィールド一覧
        var list = eval(keys.innerHTML);

        // 項目
        var obj;

        for (var i = 0 ; i < list.length; i++)
        {
            obj = Commons.getLayerObject('self', list[i]);
            if (obj) {

                if (obj.id == pmKey) {
                    return true;
                }
            }
        }

    } else {

        throw '[実装の問題]メッセージ対象項目の返却エリア「' + MESSAGE + '_FIELD」が存在しません';
    }
    return false;
};

/**
 * メッセージの有無を判定。
 * @return 判定結果(true あり or false なし)
 */
USAX.hasMessage = function()
{
    // メッセージ・オブジェクト
    var msgTxt = Commons.getLayerObject('self', MESSAGE + '_TEXT');

    if (msgTxt) {

        if (msgTxt.innerHTML) {
            return (0 < msgTxt.innerHTML.length);
        }

    } else {

        throw '[実装の問題]メッセージの返却エリア「' + MESSAGE + '_TEXT」が存在しません';
    }
};

/**
 * エラーの有無を判定。
 * @return 判定結果(true あり or false なし)
 */
USAX.hasError = function()
{
    // メッセージ・オブジェクト
    var msgTxt = Commons.getLayerObject('self', MESSAGE + '_RESULT');

    if (msgTxt) {

        if (msgTxt.innerHTML) {

            // 処理結果
            var result = msgTxt.innerHTML;

            return (result == 'ERROR' || result == 'ERROR/NEXTVIEW');
        }

    } else {

        throw '[実装の問題]処理結果の返却エリア「' + MESSAGE + '_RESULT」が存在しません';
    }
};

/**
 * メッセージを表示。
 */
USAX.printMessage = function()
{
    // メッセージ・表示ボタン
    var msgBtn = Commons.getLayerStyle('self', MESSAGE + '_OPENER');
    if (msgBtn) {
        msgBtn.visibility = 'hidden';
        msgBtn.display = 'none';
    }

    var msgWin = Commons.getLayerStyle('self', MESSAGE);
    if (msgWin) {

        if (USAX.hasMessage()) {

            msgWin.visibility = 'visible';
            msgWin.display = 'block';
            window.scrollTo(0, 0);

            if (Design) Design.boundIn(MESSAGE, 0);

        } else {

            msgWin.visibility = 'hidden';
            msgWin.display = 'none';
        }
    }

    USAX.changeMessageTargetColor(COLOR_ERROR);
};

/**
 * メッセージを隠す。
 */
USAX.closeMessage = function()
{
    var msgBtn = Commons.getLayerStyle('self', MESSAGE + '_OPENER');

    if (msgBtn) {

        if (USAX.hasMessage()) {
            msgBtn.visibility = 'visible';
            msgBtn.display = 'block';
        }
    }

    var msgWin = Commons.getLayerStyle('self', MESSAGE);
    if (msgWin) {
        msgWin.visibility = 'hidden';
        msgWin.display = 'none';
    }
};


// ############################################################################
// 入力項目の制御
// ############################################################################

/**
 * 入力項目のフォーカス時の処理。
 * @param  pmObj 入力項目
 */
USAX.setFocus = function(pmObj)
{
    if (!(USAX.isMessageTargetColor(pmObj.id))) {
        Commons.setBgColor(pmObj.id, COLOR_FOCUS);
    }
};

/**
 * 入力項目のブラー時の処理。
 * @param  pmObj 入力項目
 */
USAX.setBlur = function(pmObj)
{
    if (!(USAX.isMessageTargetColor(pmObj.id))) {
        Commons.setBgColor(pmObj.id, '');
    }
};

/**
 * 入力項目の更新の有無を判定。
 * @return 判定結果(true:更新あり / false:更新なし)
 */
USAX.isUpdate = function()
{
    var obj = Commons.getLayerObject('self', 'USAX_IS_UPDATED');

    if (obj) {
        return (obj.value == '1');
    }
    return false;
};

/**
 * 入力項目の更新の有無を設定。
 * @param  pmFlag フラグ(true:更新あり / false:更新なし)
 */
USAX.setUpdate = function(pmFlag)
{
    var obj = Commons.getLayerObject('self', 'USAX_IS_UPDATED');

    if (obj) {

        if (pmFlag) {
            obj.value = '1';
        } else {
            obj.value = '0';
        }
    }
};



// ############################################################################
// イベントの制御
// ############################################################################
/**
 * 指定フレームにイベントを送信する(Foreground)。
 * @param  pmEvent イベント名
 * @param  pmFrame フレーム名
 * @param  pmH     縦サイズ
 * @param  pmW     横サイズ
 */
USAX.doSubmit = function(pmEvent, pmFrame, pmH, pmW)
{
    // Event
    var event = document.createElement('input');

    // Frame
    var frame = document.forms[0].target;

    if (HTTPRequest.isLocked()) return;

    HTTPRequest.lock();

    try {

        // ============================================================================================
        // 画面の初期化
        // ============================================================================================
        // フィールドの色付けを解除
        USAX.changeMessageTargetColor('');
    
        // メッセージを閉じる
        USAX.closeMessage();
    
        // カーソルを変更する
        Commons.changeCursor('wait');
    
    
        // デバッグ画面を閉じる(エラー時は気にしない)
        try { ScreenDebugger.closeDebug(); } catch(wr) { }
    
        // ============================================================================================
        // イベント送信前処理
        // ============================================================================================
        if (self.preSubmit) {

            if (!(self.preSubmit(pmEvent))) {
                return false;
            }
        }
    
        // ============================================================================================
        // ウインドウの準備
        // ============================================================================================
        // オプション
        var opt= 'location=no,menubar=no,toolbar=no,status=no,resizable=yes,scrollbars=yes';

        // 送信先
        var target;

        if (pmFrame) {

            target = pmFrame;

            if (pmH || pmW) {
                opt += ',height=' + pmH + ',width=' + pmW;
            }

        } else {

            target = 'SUB' + new Date().getTime();
        }
        HTTPRequest.openWindow(target, 'about:blank', opt);

        // ============================================================================================
        // イベントの送信
        // ============================================================================================
        event.setAttribute('name', 'USAX_EVENT_EXEC');
        event.setAttribute('type', 'hidden');
        event.setAttribute('value', pmEvent);
        document.forms[0].appendChild(event);
        document.forms[0].target = target;
        document.forms[0].submit();

    } finally {

        document.forms[0].removeChild(event);
        document.forms[0].target = frame;

        // カーソルを戻す
        Commons.changeCursor('default');
        HTTPRequest.unlock();
    }
};


/**
 * 自フレームにイベントを送信する(Background)。
 * @param  pmEvent イベント名
 * @param  pmType 通信種別
 */
USAX.callEvent = function(pmEvent, pmType)
{
    // ============================================================================================
    // 画面のロック開始
    // ============================================================================================
    if (HTTPRequest.isLocked()) return;

    HTTPRequest.lock();

    // ============================================================================================
    // 画面の初期化
    // ============================================================================================
    // フィールドの色付けを解除
    USAX.changeMessageTargetColor('');

    // メッセージを閉じる
    USAX.closeMessage();

    // カーソルを変更する
    Commons.changeCursor('wait');

    // デバッグ画面を閉じる(エラー時は気にしない)
    try { ScreenDebugger.closeDebug(); } catch(wr) { }

    // ============================================================================================
    // イベント送信前処理
    // ============================================================================================
    if (self.preSubmit) {

        if (!(self.preSubmit(pmEvent))) {

            // カーソルを戻す
            Commons.changeCursor('default');
            HTTPRequest.unlock();
            return false;
        }
    }

    // ============================================================================================
    // イベントの送信
    // ============================================================================================
    var parameter = {'USAX_EVENT_EXEC':pmEvent};

    if (pmType) {
        parameter['USAX_RESPONSE_TYPE'] = pmType;
    } else {

        if (pmEvent.indexOf('BTN_DOWNLOAD') != -1) {
            parameter['USAX_RESPONSE_TYPE'] = 'blob';
        }
    }

    HTTPRequest.connect(
     document.forms[0].action,
     parameter,
     function(e)
     {
         if (this.readyState == 4) {
         // 処理終了後の処理

             if (this.responseType == 'blob') {
             // Blob

                 // ===============================================================================
                 // ファイル名の取得
                 // ===============================================================================

                 // ファイル名
                 var filename = "";

                 var disposition = this.getResponseHeader('Content-Disposition');

                 if (disposition) {

                     // ファイル名抽出のための正規表現
                     var filenameRegex = /filename[^;=\n]*=((['"]).*?\2|[^;\n]*)/;

                     // 抽出結果
                     var matches = filenameRegex.exec(disposition);

                     if (matches && matches[1]) { 
                         filename = matches[1].replace(/['"]/g, '');
                     }
                 }

                 // ===============================================================================
                 // ファイルの出力
                 // ===============================================================================

                 // Blobオブジェクトにファイルを格納
                 var blob = new Blob([this.response]);

                 if (window.navigator.msSaveBlob) {
                 // IEとEdge
                     window.navigator.msSaveBlob(blob, filename);
                 } else {
                 // それ以外

                     // Link
                     var link = document.createElement("a");

                     link.href = window.URL.createObjectURL(blob);
                     link.download = filename;

                     document.body.appendChild(link);
                     link.click();
                     document.body.removeChild(link);
                 }

             } else {
             // HTML
                    
                 // 処理結果
                 var html = this.responseText;
    
                 // 事前処理
                 if (self.preCallBack) {
                     self.preCallBack(pmEvent, html);
                 }
    
                 // 事後処理
                 if (self.callBack) {
                     if (this.status == 200) {
                         self.callBack(pmEvent, html);
                     }
                 }
             }

             // 画面ロック解除
             HTTPRequest.unlock();

             Commons.changeCursor('default');
         }
     });
};

/**
 * 一覧のページを変更する。
 * @param  pmList リストID
 * @param  pmPage ページNO
 */
USAX.movePage = function(pmList, pmPage)
{
    Commons.setFormValue('self', pmList + '-USAX_PAGE', pmPage);

    USAX.callEvent('REWRITE:BLOCK_' + pmList);
};

/**
 * 一覧の行数を変更する。
 * @param  pmList リストID
 * @param  pmRows 行数
 */
USAX.rows = function(pmList, pmRows)
{
    Commons.setFormValue('self', pmList + '-USAX_ROWS', pmRows);

    USAX.callEvent('REWRITE:BLOCK_' + pmList);
};

/**
 * 一覧のソートを変更する。
 * @param  pmList リストID
 * @param  pmSort ソート順
 */
USAX.sort = function(pmList, pmSort)
{
    Commons.setFormValue('self', pmList + '-USAX_SORT', pmSort);

    USAX.callEvent('REWRITE:BLOCK_' + pmList);
};

/**
 * コールバック処理前の処理を実行する。
 * (共通的な処理で実装負担を軽減)
 * @param  pmEvent イベント名
 * @param  pmHTML  結果HTML
 * @return callBackの実行有無(true あり or false なし)
 */
function preCallBack(pmEvent, pmHTML)
{
    // ========================================================================
    // メッセージ・ブロックを書換え
    // ========================================================================

    // 実行前のシステム制御情報
    var system_bak = Commons.getLayerObject('self', 'USAX_SYSTEM_CONTROL');

    // 結果の反映
    Commons.changeTagById(pmHTML, MESSAGE);

    // 実行後のシステム制御情報
    var system_new = Commons.getLayerObject('self', 'USAX_SYSTEM_CONTROL');

    // 制御情報が取得可能かチェックする
    if (system_new) {

        if (system_new.innerHTML.replace(/^\s+|\s+$/g, '') != '') {
            system_bak = '';
        }
    }

    // 制御情報が取得できない場合は実行前の制御情報をダメ元で復元する
    if (system_bak) Commons.replaceTag(system_new, system_bak);

    // ========================================================================
    // 処理結果の確認と画面描画
    // ========================================================================

    // メッセージ・オブジェクト
    var msgTxt = Commons.getLayerObject('self', MESSAGE + '_RESULT');

    // 処理結果
    var result = 'SUCCESS';

    var ret = false;

    if (!msgTxt) throw 'No processing result - ' + MESSAGE + '_RESULT';

    // 処理結果
    result = msgTxt.innerHTML;

    if (result) {

        if (result == 'SUCCESS/NEXTVIEW' || result == 'ERROR/NEXTVIEW') {
        // 画面遷移(画面全体を書換え)

            Commons.changeTag(pmHTML, 'head');
            Commons.changeTag(pmHTML, 'body');
            USAX.pushState();

            ret = (result.indexOf('SUCCESS/NEXTVIEW:') == 0);

        } else
        if ((result.indexOf('SUCCESS/REWRITE:') == 0)
         || (result.indexOf('ERROR/REWRITE:')   == 0)) {
        // 画面再描画

            ret = (result.indexOf('SUCCESS/REWRITE:') == 0);

            result = result.replace('SUCCESS/REWRITE:', '');
            result = result.replace('ERROR/REWRITE:',   '');

            // 書換え項目ID一覧
            var list = result.split(',');

            for (var i = 0; i < list.length; i++)
            {
                if (list[i].toUpperCase() == 'ALL') {
                    Commons.changeTag(pmHTML, 'head');
                    Commons.changeTag(pmHTML, 'body');
                } else
                if (list[i].toUpperCase() == 'HEAD') {
                    Commons.changeTag(pmHTML, 'head');
                } else
                if (list[i].toUpperCase() == 'BODY') {
                    Commons.changeTag(pmHTML, 'body');
                } else {

                    Commons.changeTagById(pmHTML, list[i]);

                    if (list[i] == 'SCREEN_DEBUG') {
                       // デバッグ機能の更新の場合

                        // デバッグ表示を有効にする
                        Commons.setBgColor('BTN_SCREEN_DEBUGGER', '#AA0000');
                        ScreenDebugger._visibleScreenDebug = true;

                        Commons.setDisplay('SCREEN_DEBUG', 'block');
                    }
                }
            }

        } else
        if (result.indexOf('SUCCESS') == 0) {
            ret = true;
        }
    }

    // ========================================================================
    // 画面再描画イベントの処理
    // ========================================================================
    Commons.changeCursor('default');

    if (result == 'SUCCESS/NEXTVIEW' || result == 'ERROR/NEXTVIEW') {
    // 画面遷移

        // 画面表示時の初期処理
        if (self.doInitialize) {
            setTimeout("Commons.checkLoading('self.doInitialize()');", 5);
        }

    } else {
    // 書換え

        // 画面書換え時の処理
        if (self.doRewrite) {
            setTimeout("Commons.checkLoading('self.doRewrite()');", 5);
        }
    }
    return ret;
}


/**
 * キーボード押下時のイベント。
 */
window.document.onkeydown = function()
{
    try {

        if (window.event.keyCode == 13) {
        // 改行

            if (window.event.srcElement.tagName.toUpperCase() == 'TEXTAREA') {
            // TEXTAREAの場合

                return true;

            } else
            if (window.event.srcElement.tagName.toUpperCase() == 'BUTTON') {
                window.event.srcElement.cleck();
            } else {
                Commons.nextFocus(window.event.srcElement);
            }
            return false;

        } else
        if (window.event.keyCode ==  8) {
        // バックスペース

            if (window.event.srcElement.readOnly == true) {
            // ReadOnly項目上でのバックスペース(画面HistoryBack)の禁止

                // onClickイベントがあれば代わりに呼び出す
                if (window.event.srcElement.onclick) {
                    window.event.srcElement.onclick();
                }
                return false;
            }
        }

    } catch (ex) {
        try { console.log(ex); } catch (wr) { }
    }
};

/**
 * キーボード解放時のイベント。
 */
window.document.onkeyup = function()
{
    try {

        if (window.event.ctrlKey || window.event.altKey) return;

        if (( 48 <= window.event.keyCode && window.event.keyCode <=  57)
         || ( 65 <= window.event.keyCode && window.event.keyCode <=  90)
         || ( 96 <= window.event.keyCode && window.event.keyCode <= 105)
         || (106 <= window.event.keyCode && window.event.keyCode <= 107)
         || (109 <= window.event.keyCode && window.event.keyCode <= 111)
         || (186 <= window.event.keyCode && window.event.keyCode <= 192)
         || (219 <= window.event.keyCode && window.event.keyCode <= 222)
         || (window.event.keyCode == 226)
         || (window.event.keyCode ==   8 || window.event.keyCode ==  32 || window.event.keyCode ==  46)
         || (window.event.keyCode ==  17)) {

            if (self.doJustInTime) {

                doJustInTime(window.event.srcElement.id);
            }
        }

    } catch (ex) {
        try { console.log(ex); } catch (wr) { }
    }
};
